# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='App',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('status', models.CharField(max_length=255)),
                ('category', models.CharField(max_length=255)),
                ('source', models.CharField(blank=True, max_length=600, null=True)),
                ('repository', models.CharField(blank=True, max_length=600, null=True)),
                ('install', models.CharField(blank=True, max_length=600, null=True)),
                ('main', models.CharField(blank=True, max_length=600, null=True)),
                ('type', models.CharField(max_length=255)),
                ('config', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Create your tasks here
from __future__ import absolute_import, unicode_literals
from celery import shared_task

from tasks.models import Task
from .models import Analysis
from samples.models import Sample, SampleGroup

from celery import Celery
app = Celery('mendelmd')

# Using a string here means the worker will not have to
# pickle the object when using Windows.
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)

@app.task(queue="master")
def create_analysis_tasks(analysis_id):
    print('analysis_id', analysis_id)
    # samples = []
    # print('hello!')
    #create analysis tasks
    # analysis = Analysis.objects.()
    analysis = Analysis.objects.get(pk=analysis_id)
    print(dir(analysis))
    params = analysis.params
    files = params['files']
    for file in files:
        task = Task(user=analysis.user)
        task.manifest = {}
        task.manifest['files'] = [file]
        task.manifest['analysis_types'] = params['analysis_types']
        task.status = 'new'
        task.analysis = analysis
        task.action = 'analysis'
        task.save()
        analysis.tasks.add(task)

    # if 'sample_groups' in  params:
    #     samples = Sample.objects.filter(samplegroup_members__in=params['sample_groups'])
    #     # sample = Sample.objects.first()
    #     # print(dir(sample))
    # for sample in samples:      
    #     print(sample)
    #     for file in sample.files.all():
    #         bam_size = 9223372036854775807
    #         if file.extension == 'bam':
    #             if file.size < bam_size:
    #                 bamfile = file
    #                 bam_size = file.size
    #     print('small bam', bamfile.size)
    # get smallest bam file


from django.db import models
from django.forms import ModelForm
from .models import Project
from django import forms

class ProjectForm(ModelForm):
    class Meta:
        model = Project
        fields = ['name', 'description', 'groups']#, 'members', #'paths', 
        # fields = '__all__'

class ImportForm(forms.Form):
    # paths = forms.CharField(widget=forms.Textarea, required=False)
    file_list = forms.CharField(widget=forms.Textarea, required=False)
    samples = forms.CharField(widget=forms.Textarea, required=False)


from django.conf.urls import *

from . import views

urlpatterns = [
    url(r'^$', views.index, name='variant_index'),
    url(r'^view/(?P<variant_id>\d+)/$', views.view, name='variant_view'),
    
]


from django import forms
from genes.models import GeneGroup
from django.forms import ModelForm

class GeneGroupForm(ModelForm):
    class Meta:
        model = GeneGroup
        fields = '__all__'


class GenesetForm(forms.Form):
  name = forms.CharField(max_length=255)
  genes = forms.CharField(widget=forms.widgets.Textarea())

  def __init__(self, *args, **kwargs):
      self.instance = kwargs.pop('instance', None)
      super(GenesetForm, self).__init__(*args, **kwargs)
      # self.fields['tags'].choices = [(tag.id, tag.title) for tag in Tag.objects]
      if self.instance:
          self.fields['name'].initial = self.instance.name
          self.fields['genes'].initial = self.instance.genes
          # self.fields['is_published'].initial = self.instance.is_published        
          # self.fields['tags'].initial = [tag.id for tag in self.instance.tags]


  


from django.core.management.base import BaseCommand, CommandError

from django.db import connection
        
class Command(BaseCommand):
    # args = '<poll_id poll_id ...>'
    help = 'Import data from dbnfsp'

    def handle(self, *args, **options):

        files = list(range(1,23))+['X','Y','M']
        # files = ['22']
        print(files)
        for i in files:
            # command = 'wc -l dbNSFP3.5a_variant.chr{}'.format(i)
            import_file = 'dbNSFP3.5a_variant.chr{}'.format(i)
            print(i)           
            with connection.cursor() as cursor:
                # cursor.execute("UPDATE bar SET foo = 1 WHERE baz = %s", [self.baz])
                cursor.execute("""copy databases_dbnfsp ("chr","pos_1_based","ref","alt","aaref","aaalt","rs_dbSNP150","hg19_chr","hg19_pos_1_based","hg18_chr","hg18_pos_1_based","genename","cds_strand","refcodon","codonpos","codon_degeneracy","Ancestral_allele","AltaiNeandertal","Denisova","Ensembl_geneid","Ensembl_transcriptid","Ensembl_proteinid","aapos","SIFT_score","SIFT_converted_rankscore","SIFT_pred","Uniprot_acc_Polyphen2","Uniprot_id_Polyphen2","Uniprot_aapos_Polyphen2","Polyphen2_HDIV_score","Polyphen2_HDIV_rankscore","Polyphen2_HDIV_pred","Polyphen2_HVAR_score","Polyphen2_HVAR_rankscore","Polyphen2_HVAR_pred","LRT_score","LRT_converted_rankscore","LRT_pred","LRT_Omega","MutationTaster_score","MutationTaster_converted_rankscore","MutationTaster_pred","MutationTaster_model","MutationTaster_AAE","MutationAssessor_UniprotID","MutationAssessor_variant","MutationAssessor_score","MutationAssessor_score_rankscore","MutationAssessor_pred","FATHMM_score","FATHMM_converted_rankscore","FATHMM_pred","PROVEAN_score","PROVEAN_converted_rankscore","PROVEAN_pred","Transcript_id_VEST3","Transcript_var_VEST3","VEST3_score","VEST3_rankscore","MetaSVM_score","MetaSVM_rankscore","MetaSVM_pred","MetaLR_score","MetaLR_rankscore","MetaLR_pred","Reliability_index","M_CAP_score","M_CAP_rankscore","M_CAP_pred","REVEL_score","REVEL_rankscore","MutPred_score","MutPred_rankscore","MutPred_protID","MutPred_AAchange","MutPred_Top5features","CADD_raw","CADD_raw_rankscore","CADD_phred","DANN_score","DANN_rankscore","fathmm_MKL_coding_score","fathmm_MKL_coding_rankscore","fathmm_MKL_coding_pred","fathmm_MKL_coding_group","Eigen_coding_or_noncoding","Eigen_raw","Eigen_phred","Eigen_PC_raw","Eigen_PC_phred","Eigen_PC_raw_rankscore","GenoCanyon_score","GenoCanyon_score_rankscore","integrated_fitCons_score","integrated_fitCons_score_rankscore","integrated_confidence_value","GM12878_fitCons_score","GM12878_fitCons_score_rankscore","GM12878_confidence_value","H1_hESC_fitCons_score","H1_hESC_fitCons_score_rankscore","H1_hESC_confidence_value","HUVEC_fitCons_score","HUVEC_fitCons_score_rankscore","HUVEC_confidence_value","GERP_NR","GERP_RS","GERP_RS_rankscore","phyloP100way_vertebrate","phyloP100way_vertebrate_rankscore","phyloP20way_mammalian","phyloP20way_mammalian_rankscore","phastCons100way_vertebrate","phastCons100way_vertebrate_rankscore","phastCons20way_mammalian","phastCons20way_mammalian_rankscore","SiPhy_29way_pi","SiPhy_29way_logOdds","SiPhy_29way_logOdds_rankscore","Gp3_AC_1k","Gp3_AF_1k","Gp3_AFR_AC_1k","Gp3_AFR_AF_1k","Gp3_EUR_AC_1k","Gp3_EUR_AF_1k","Gp3_AMR_AC_1k","Gp3_AMR_AF_1k","Gp3_EAS_AC_1k","Gp3_EAS_AF_1k","Gp3_SAS_AC_1k","Gp3_SAS_AF_1k","TWINSUK_AC","TWINSUK_AF","ALSPAC_AC","ALSPAC_AF","ESP6500_AA_AC","ESP6500_AA_AF","ESP6500_EA_AC","ESP6500_EA_AF","ExAC_AC","ExAC_AF","ExAC_Adj_AC","ExAC_Adj_AF","ExAC_AFR_AC","ExAC_AFR_AF","ExAC_AMR_AC","ExAC_AMR_AF","ExAC_EAS_AC","ExAC_EAS_AF","ExAC_FIN_AC","ExAC_FIN_AF","ExAC_NFE_AC","ExAC_NFE_AF","ExAC_SAS_AC","ExAC_SAS_AF","ExAC_nonTCGA_AC","ExAC_nonTCGA_AF","ExAC_nonTCGA_Adj_AC","ExAC_nonTCGA_Adj_AF","ExAC_nonTCGA_AFR_AC","ExAC_nonTCGA_AFR_AF","ExAC_nonTCGA_AMR_AC","ExAC_nonTCGA_AMR_AF","ExAC_nonTCGA_EAS_AC","ExAC_nonTCGA_EAS_AF","ExAC_nonTCGA_FIN_AC","ExAC_nonTCGA_FIN_AF","ExAC_nonTCGA_NFE_AC","ExAC_nonTCGA_NFE_AF","ExAC_nonTCGA_SAS_AC","ExAC_nonTCGA_SAS_AF","ExAC_nonpsych_AC","ExAC_nonpsych_AF","ExAC_nonpsych_Adj_AC","ExAC_nonpsych_Adj_AF","ExAC_nonpsych_AFR_AC","ExAC_nonpsych_AFR_AF","ExAC_nonpsych_AMR_AC","ExAC_nonpsych_AMR_AF","ExAC_nonpsych_EAS_AC","ExAC_nonpsych_EAS_AF","ExAC_nonpsych_FIN_AC","ExAC_nonpsych_FIN_AF","ExAC_nonpsych_NFE_AC","ExAC_nonpsych_NFE_AF","ExAC_nonpsych_SAS_AC","ExAC_nonpsych_SAS_AF","gnomAD_exomes_AC","gnomAD_exomes_AN","gnomAD_exomes_AF","gnomAD_exomes_AFR_AC","gnomAD_exomes_AFR_AN","gnomAD_exomes_AFR_AF","gnomAD_exomes_AMR_AC","gnomAD_exomes_AMR_AN","gnomAD_exomes_AMR_AF","gnomAD_exomes_ASJ_AC","gnomAD_exomes_ASJ_AN","gnomAD_exomes_ASJ_AF","gnomAD_exomes_EAS_AC","gnomAD_exomes_EAS_AN","gnomAD_exomes_EAS_AF","gnomAD_exomes_FIN_AC","gnomAD_exomes_FIN_AN","gnomAD_exomes_FIN_AF","gnomAD_exomes_NFE_AC","gnomAD_exomes_NFE_AN","gnomAD_exomes_NFE_AF","gnomAD_exomes_SAS_AC","gnomAD_exomes_SAS_AN","gnomAD_exomes_SAS_AF","gnomAD_exomes_OTH_AC","gnomAD_exomes_OTH_AN","gnomAD_exomes_OTH_AF","gnomAD_genomes_AC","gnomAD_genomes_AN","gnomAD_genomes_AF","gnomAD_genomes_AFR_AC","gnomAD_genomes_AFR_AN","gnomAD_genomes_AFR_AF","gnomAD_genomes_AMR_AC","gnomAD_genomes_AMR_AN","gnomAD_genomes_AMR_AF","gnomAD_genomes_ASJ_AC","gnomAD_genomes_ASJ_AN","gnomAD_genomes_ASJ_AF","gnomAD_genomes_EAS_AC","gnomAD_genomes_EAS_AN","gnomAD_genomes_EAS_AF","gnomAD_genomes_FIN_AC","gnomAD_genomes_FIN_AN","gnomAD_genomes_FIN_AF","gnomAD_genomes_NFE_AC","gnomAD_genomes_NFE_AN","gnomAD_genomes_NFE_AF","gnomAD_genomes_OTH_AC","gnomAD_genomes_OTH_AN","gnomAD_genomes_OTH_AF","clinvar_rs","clinvar_clnsig","clinvar_trait","clinvar_golden_stars","Interpro_domain","GTEx_V6p_gene","GTEx_V6p_tissue") from '/storage3/dev/dbnfsp/dbNSFP3.5a_variant.chr{}' delimiter E'\t' csv header;""".format(i))
            # run(command, shell=True)





from django.http import HttpResponseRedirect
from django.http import HttpResponse
from django.shortcuts import render_to_response, redirect
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
#from forms import PathwayAnalysisForm
from django.shortcuts import render_to_response, get_object_or_404
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.db.models import Q
from django.db.models import Count, Sum, Avg, Min, Max
import csv
from django.contrib import messages
from django.core import serializers
from django.core.urlresolvers import reverse 

from django.views.generic import UpdateView, DeleteView

from django.shortcuts import render_to_response
#from django.contrib.formtools.wizard import FormWizard
from django.contrib.formtools.wizard.views import SessionWizardView

from pathway_analysis.forms import PathwayAnalysisForm, PathAnalysisForm

from pathway_analysis.models import Pathway
from filter_analysis.models import *
from filter_analysis.forms import *


from genes.models import GeneGroup, Gene, CGDEntry, GeneList

from diseases.models import Disease, HGMDGene, HGMDPhenotype
from diseases.models import Gene as GeneDisease


#pathwya imports
from SOAPpy import WSDL
#REST
import urllib.request, urllib.error, urllib.parse


def kegg_rest_request(query):
    url = 'http://rest.kegg.jp/%s' % (query)
    print(url)
    try:
        data = urllib.request.urlopen(url).read()
    except urllib.error.HTTPError as e:
        print("HTTP error: %d" % e.code)
    except urllib.error.URLError as e:
        print("Network error: %s" % e.reason.args[1])
    
    return data

def parse_pathways(data):
    data = data.strip().split('\n')
    pathways = {}
    for line in data:
        line = line.split('\t')
        print(line)
        line[0] = line[0].replace('path:', '').replace('hsa', '').replace('map', '')
        pathways[line[0]] = line[1]
    return pathways

def parse_genes(data):
    #get a gene http://rest.kegg.jp/get/hsa:10683
    data = data.strip().split('\n')
    genes = []
    flag = False
    for line in data:
        if not line.startswith('GENE'):
            if line[0].isupper():
                flag = False
        if line.startswith('GENE') or flag:
            if line.startswith('GENE'):
                line = line.replace('GENE        ', '')
            else:
                line = line.strip()
            
            flag =  True
            
            line = line.split()
            
            #print 'line', line

            gene = {}
            
            
            if len(line) > 1:
                gene['id'] = line[0]
                gene['symbol'] = line[1].replace(';', '')
                gene['ko'] = line[-1]
                gene['description'] = " ".join(line[2:-2])
                genes.append(gene) 
            
            
            
            
        
        
    return genes
        

def populate(request):
#    pathway = Pathway(keggid='1234', name='Raony', genes='ABGS,SAKHA,AKJSKJ')
#    pathway.save()

    Pathway.objects.all().delete()

    pathways = kegg_rest_request('list/pathway/hsa')
    pathways = parse_pathways(pathways)
    insert_list = []
    total = len(pathways)
    
    for keggid in pathways:
        print(total)
        total = total-1
        
#        print keggid, pathways[keggid]
        pathway_data = kegg_rest_request('get/hsa%s' % (keggid))
        genes = parse_genes(pathway_data)
        print('genes')
#        print genes
        gene_list = []
        for gene in genes:
            gene_list.append(gene['symbol'])
#        print gene_list
        gene_list = ",".join(gene_list)
#        print keggid, pathways[keggid], gene_list
#        pathway_name = pathway_data.split('\n')[1].replace('NAME', '')
    
        pathway = Pathway(kegg=keggid, name=pathways[keggid], genes=gene_list)
        pathway.save()

    message = 'finish filling pathway from kegg'
    messages.add_message(request, messages.INFO, message)
    return redirect('/databases/')


# Create your views here.
@login_required
def index(request):
    if request.method == 'POST':
        form = PathAnalysisForm(request.POST)
        if  form.is_valid():
            query = form.cleaned_data['search']
            print(query)
            #here is where the magic happens!
            #search in kegg
#            data = kegg_rest_request('list/pathway/hsa')
#            pathways = kegg_rest_request('find/pathway/%s' % (query))
            pathways = Pathway.objects.filter(Q(name__icontains=query))
            # print pathways
            
    else:
        form = PathAnalysisForm()
#        pathways = kegg_rest_request('list/pathway/hsa')
        pathways = Pathway.objects.all()    
    
    
    return render_to_response('pathway_analysis/index.html', {'form': form, 'pathways': pathways}, context_instance=RequestContext(request))

@login_required
def view(request, pathway_id):
    print(pathway_id)
    
#    pathway_data = kegg_rest_request('get/hsa%s' % (pathway_id))
    pathway = Pathway.objects.get(kegg=pathway_id)

     
    pathway.genes = pathway.genes.split(',')
    print(pathway.genes)
    
#    genes = parse_genes(pathway_data)
#    pathway = {}
#    pathway['name'] = pathway_data.split('\n')[1].replace('NAME', '') 
#    #get gene_ids
#    genelist = []
#    for gene in genes:
#        
#        genelist.append('hsa:%s' % gene['id'])
##        print gene['id']
#    gene_url = '+'.join(genelist)
#    url = '/conv/ncbi-geneid/%s' % (gene_url)
#    results = kegg_rest_request(url)
    #print results
    
    
    #if request.method == 'GET':
    return render_to_response('pathway_analysis/view.html', {'pathway':pathway}, context_instance=RequestContext(request))

from filter_analysis.views import filter_analysis

#This is the most important function
@login_required
def analysis(request):
    query = {}
    exclude = {}
    summary = {}
    args = []
    results = []
    
    query_string = request.META['QUERY_STRING']
    new_query_string = []
    for item in query_string.split('&'):
        if not item.startswith('page'):
             new_query_string.append(item)
    query_string = "&".join(new_query_string)
    
    print('query string')
    print(query_string) 
    filteranalysis = FilterAnalysis.objects.all().prefetch_related('user')
    filterconfigs = FilterConfig.objects.all().prefetch_related('user')
    if query_string != [''] and query_string != '' :
        if request.method == 'GET':
            
            form = PathwayAnalysisForm(request.GET)
            
            if 'sort_by' in request.GET:
                sort_by = request.GET['sort_by']
                if sort_by == 'desc':
                    sort = 'asc'
                else:
                    sort = 'desc'
            else:
                sort = 'asc'

            if 'order_by' in request.GET:
                if sort_by == 'desc':
                    order_by = '-%s' % request.GET['order_by']
                else:
                    order_by = request.GET['order_by']
            else:
                order_by = 'gene'

            filter_analysis(request,query, args, exclude)

            # #CHR
            # filter_chr(request, query)
            # #pos
            # filter_pos(request, query)
            # #snp_list
            # filter_snp_list(request, query, exclude)
            # #Gene List, this is entered by hand
            # filter_gene_list(request, query, args)
            # #Inheritance_model
            # filter_mutation_type(request, args)
            # #Clinical
            # # filter_cln(request, query)
            # #Variant type SNP EFF       
            # filter_variant_type_snpeff(request, query)
            # #DBSNP
            # filter_dbsnp(request, query)
            # #1000genomes
            # filter_by_1000g(request, args)
            # #dbsnp Freq
            # filter_by_dbsnp(request, args)
            # #Exome Variation Server Freq
            # filter_by_esp(request, args)
            # #hi score
            # filter_by_hi_score(request, args)
            # #SIFT
            # filter_by_sift(request, args)
            # #POLYPHEN
            # filter_by_pp2(request, args)
            # filter_by_segdup(request, args)
            # #filter by disease databases
            # filter_omim(request, args)
            # filter_cgd(request, args)
            # filter_hgmd(request, args)
            # filter_genelists(request, query, args, exclude)
            # #DBSNP Build
            # filter_dbsnp_build(request, query)            
            # #Read Depth
            # filter_read_depth(request, args)            
            # filter_qual(request, args)
            # filter_filter(request, query)
            # #Functional class
            # filter_func_class(request, query)
            # #IMPACT
            # filter_impact(request, query)

            #exclude individuals
            exclude_individuals = request.GET.getlist('exclude_individuals')
#            print exclude_individuals
            
            #exclude groups append to individuals
            exclude_groups = request.GET.getlist('exclude_groups')
#            print exclude_groups
            exclude_individuals_list = []
            if len(exclude_groups) > 0:
                for group_id in exclude_groups:
                    group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
                    for individual in group_individuals:
                         exclude_individuals_list.append(str(str(individual)))
#                    print exclude_individuals_list
                    
            exclude_individuals_list = exclude_individuals_list + exclude_individuals
#            print exclude_individuals_list
            exclude_individuals_variants = {}
            #exclude variants from individuals
            if len(exclude_individuals_list) > 0:
                
                #for all individuals in exclude list
                for individual in exclude_individuals_list:
                    individual_variants = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values('chr', 'pos', 'genotype')
                    for variant in individual_variants:
                        id = '%s-%s' % (variant['chr'], variant['pos'])                        
                        if id in exclude_individuals_variants:
                            exclude_individuals_variants[id][variant['genotype']] = 0
                        else:
                            exclude_individuals_variants[id] = {}
                            exclude_individuals_variants[id][variant['genotype']] = 0
                            
            #INDIVIDUALS (MUST BE THE LAST ONE...) WHY ???
            individuals = request.GET.getlist('individuals')
            
            groups = request.GET.getlist('groups')
            print(groups)
            individuals_list = []
            if len(groups) > 0:
                for group_id in groups:
                    group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
                    for individual in group_individuals:
                         individuals_list.append(str(str(individual)))
                    print(individuals_list)
                    
            individuals_list = individuals_list + individuals
#            print 'Hello World Raony'
            if len(individuals_list) > 0:
                query['individual_id__in'] = individuals_list
                #exclude variants from individuals_list
                if len(exclude_individuals_list) > 0:
    #                print 'now excluding variants from exclude_individuals'
                    variants_ids = []
                    #get variants from all select individuals    
                    variants = Variant.objects.filter(*args, **query).exclude(**exclude).values('id', 'chr', 'pos', 'genotype')
                    for variant in variants:
                        id = '%s-%s' % (variant['chr'], variant['pos'])
                        if id in exclude_individuals_variants:
                            if not variant['genotype'] in exclude_individuals_variants[id]:
                                variants_ids.append(variant['id'])
                        else:
                            variants_ids.append(variant['id'])
    #                print len(variants_ids)
                    query['pk__in'] = variants_ids
                
                
                # OPTION variants per gene
                variants_per_gene = request.GET.get('variants_per_gene')
                #option GENES in COMMON
                genes_in_common = request.GET.get('genes_in_common', '')
                
                print('variants per gene')
                print(variants_per_gene)
                if variants_per_gene != '':
                    
                    variants_per_gene = int(variants_per_gene)
    #                print 'Variants per gene'
                    variants_per_gene_option = request.GET.get('variants_per_gene_option', '')
                    
                    print('Debugging')
                    print(variants_per_gene)
                    print(variants_per_gene_option)
                    genes_exclude_list = []
                    genes_only_list = []
                    #this works for 2 indivuduals in list and with show only genes with variants in both option on
                    for individual in individuals_list:
#                        print individual
#                        print query
#                        print exclude
                        print('get list of all genes for each individual')
                        individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values('gene').exclude(gene="").annotate(count=Count('gene')).distinct() #.aggregate(Count('gene', distinct=True))
                        print(len(individual_genes))
    #                    print len(individual_genes)
                        if variants_per_gene_option == '>':    
                            for gene in individual_genes:
                                if gene['count'] >= variants_per_gene:
                                    genes_only_list.append(gene['gene'])
                                else:
                                    if genes_in_common == 'on':
                                        genes_exclude_list.append(gene['gene'])
                                        
                        elif variants_per_gene_option == '<':                    
                            for gene in individual_genes:
                                if gene['count'] <= variants_per_gene:
                                    genes_only_list.append(gene['gene'])
                                else:
                                    if genes_in_common == 'on':
                                        genes_exclude_list.append(gene['gene'])
                        elif variants_per_gene_option == '=':
                            for gene in individual_genes:
                                if gene['count'] == variants_per_gene:
                                    genes_only_list.append(gene['gene'])
                                else:
                                    if genes_in_common == 'on':
                                        genes_exclude_list.append(gene['gene'])
                    #remove variants without gene name                    
                    args.append(Q(gene__in=genes_only_list))
                    args.append(~Q(gene__in=genes_exclude_list))
                
                if genes_in_common == 'on':
                    #get all genes from individual
                    individual_gene_list = []
                    for individual in individuals_list:
                        individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('gene', flat=True).exclude(gene="").distinct()
                        individual_genes = set(list(individual_genes))
                        individual_gene_list.append(individual_genes)
                    genes_in_common_list = set.intersection(*individual_gene_list)                    
                    query['gene__in'] = genes_in_common_list#genes_in_common_list

                
                positions_in_common = request.GET.get('positions_in_common', '')
                if positions_in_common == 'on':
                    #get all genes from individual
                    individual_positions_list = []
                    for individual in individuals_list:
                        #should be done with an index ex. 1-2835763-C-T
                        individual_positions = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('pos', flat=True).distinct()
                        individual_positions = set(list(individual_positions))
                        individual_positions_list.append(individual_positions)
                    positions_in_common_list = set.intersection(*individual_positions_list)                    
                    query['pos__in'] = positions_in_common_list#genes_in_common_list
            
            #last call to the DATABASE Finally!!!!!! 
            variants = Variant.objects.filter(*args, **query).exclude(**exclude).prefetch_related('individual').order_by(order_by)
            #export to csv
            export = request.GET.get('export', '')
            if export != '':
                if export == 'csv':
                    response = HttpResponse(mimetype='text/csv')
                    response['Content-Disposition'] = 'attachment; filename=export.csv'
                    writer = csv.writer(response)
                    
                elif export == 'txt':
                    response = HttpResponse(mimetype='text/plain')
                    response['Content-Disposition'] = 'attachment; filename=export.txt'
                    writer = csv.writer(response, delimiter='\t', quoting=csv.QUOTE_NONE)    
                writer.writerow(['Individual',
                'Chromossome', 'Variant Id', 'Pos', 'Qual', 'Ref', 'Alt', 'Genotype',
                'Genotype Info', 'Read Depth', 'Snpe Eff', 'Functional Class', 'Gene',
                'Impact', 'Variant is Clinical', '100Genomes Frequency', 'dbSNP135 Frequency',
                'ESP5400 Frequency', 'ESP5400 EA/AA/ALL', 'ESP5400 Total Allele Count',
                'SIFT', 'Polyphen2', 'dbSNP Build', 'Amino Acid Change',
                'Cdna Position', 'Granthamscore', 'Protein Position']) 
                for variant in variants:
                    writer.writerow([variant.individual, variant.chr,
                    variant.variant_id, variant.pos, variant.qual, variant.ref,
                    variant.alt, variant.genotype, variant.genotype_info,
                    variant.read_depth, variant.snp_eff,
                    variant.snp_eff_functional_class, variant.gene, variant.impact,
                    variant.dbsnp_pm, variant.genomes1k_maf, variant.dbsnp_gmaf,
                    variant.esp_maf_total, variant.ann_esp_maf, variant.tac,
                    variant.sift, variant.polyphen, variant.dbsnp_build,
                    variant.amino_acid_change, variant.cdna_position,
                    variant.granthamscore, variant.protein_position])
                return response

            summary['genes'] = variants.values_list('gene', flat=True)
            summary['genes'] = sorted(list(set(summary['genes'])))
            summary['n_genes'] =  len(summary['genes'])
            summary['has_variants'] = True
            
            if summary['n_genes'] < 500:
                genes = Gene.objects.filter(symbol__in=list(summary['genes'])).values('symbol', 'diseases').prefetch_related('diseases')                
                genes_hgmd = HGMDGene.objects.filter(symbol__in=list(summary['genes'])).prefetch_related('diseases')# | reduce(lambda x, y: x | y, [Q(aliases__icontains=word) for word in list(summary['genes'])]))
                #return Disease.objects.filter(Q(name__icontains=name)|Q(genes__icontains=name))
                #for each gene name create a query
                queries = [Q(genes__icontains=value) for value in list(summary['genes'])]
                if len(queries) > 0:
                    query = queries.pop()
                    for item in queries:
                        query |= item
                    genes_omim = GeneDisease.objects.filter(official_name__in=list(summary['genes'])).prefetch_related('diseases')#query
                else:
                    genes_omim = []
                #get genes at CGD
                genes_cgd = CGDEntry.objects.filter(GENE__in=list(summary['genes'])).prefetch_related('CONDITIONS')

            else:
                genes = []
                genes_hgmd = []
                genes_omim = []
                genes_cgd = []

            summary['n_variants'] = len(variants)

            #Do pathway analysis here
            pathways = Pathway.objects.all().order_by('name')

            for pathway in pathways:
                pathway.variants = []
                pathway.genes = pathway.genes.split(',')

            #now group variants object by pathways
            if variants != None:
                #create dict
                variants_dicts = {}
                for variant in variants:
                    for pathway in pathways:
                        if variant.gene in pathway.genes:
                            pathway.variants.append(variant)
                            if pathway not in results:
                                results.append(pathway)
                pathways = sorted(results, key=lambda k: k.name)
            else:
                pathways = []            

            # Filter option for pathways for individuals_per_pathway
            #check for each pathway if there (<=,>=,==) at least n individuals per pathway
            individuals_per_pathway = request.GET.get('individuals_per_pathway')
            not_allowed = ['', None]
            if individuals_per_pathway not in not_allowed:
                new_pathways = []
                individuals_per_pathway = int(individuals_per_pathway)
                print('Individuals per pathway')
                individuals_per_pathway_option = request.GET.get('individuals_per_pathway_option', '')
                
                for pathway in pathways:
    #                print pathway.name
                    #count individuals
                    individuals_in_pathway = []
                    
    #                print len(pathway.variants)
                    for variant in pathway.variants:
                        if variant.individual not in individuals_in_pathway:
                            individuals_in_pathway.append(variant.individual)
                            
                    #check size of individuals_in_pathway:
    #                print 'individuals_in_pathway'
                    if individuals_per_pathway_option == '>':
                        if len(individuals_in_pathway) >= individuals_per_pathway:
                            new_pathways.append(pathway)
                pathways = new_pathways




            
    #first entrance in methods
    else:
        pathways = []
        summary = {}
        summary['has_variants'] = False
        #fake number just not to open accordion on view 
        summary['n_genes'] = 1000
        genes = []
        genes_hgmd = []
        genes_omim = []
        genes_cgd = []
        
        form = PathwayAnalysisForm()                    
    return render_to_response('pathway_analysis/analysis.html', 
        {'pathways':pathways,
        'form':form, 
        'summary':summary, 
        'query_string':query_string, 
        'filteranalysis':filteranalysis, 
        'filterconfigs':filterconfigs, 
        'genes':genes, 
        'genes_hgmd':genes_hgmd,
        'genes_omim':genes_omim,
        'genes_cgd':genes_cgd}, context_instance=RequestContext(request))



@login_required
def index(request):
    args = []
    query = {}
    summary= {}
    results = []

    if request.method == 'GET':
        
        #prepare pathway form
        form = PathwayAnalysisForm(request.GET)
        
        #call method get_filtered_variants

        result = get_filtered_variants(request)

        if result:
            variants = result['variants']
            summary = result['summary']
        else:
           variants=None 
        
        #get all pathways for pathway analysis
        pathways = Pathway.objects.all().order_by('name')

        # all_genes_from_pathways = []
        # for pathway in pathways:
        #     pathway.variants = []
        #     pathway.genes = pathway.genes.split(',')
            # all_genes_from_pathways = all_genes_from_pathways + pathway.genes
        # print len(all_genes_from_pathways)


        #now group variants object by pathways
        if variants != None:
            #create dict
            variants_dicts = {}
            for variant in variants:
                for pathway in pathways:
                    if variant.gene in pathway.genes:
                        pathway.variants.append(variant)
                        if pathway not in results:
                            results.append(pathway)
            pathways = sorted(results, key=lambda k: k.name)
        else:
            pathways = []

        

        # Filter option for pathways for individuals_per_pathway
        #check for each pathway if there (<=,>=,==) at least n individuals per pathway
        individuals_per_pathway = request.GET.get('individuals_per_pathway')
        not_allowed = ['', None]
        if individuals_per_pathway not in not_allowed:
            new_pathways = []
            individuals_per_pathway = int(individuals_per_pathway)
            print('Individuals per pathway')
            individuals_per_pathway_option = request.GET.get('individuals_per_pathway_option', '')
            
            for pathway in pathways:
#                print pathway.name
                #count individuals
                individuals_in_pathway = []
                
#                print len(pathway.variants)
                for variant in pathway.variants:
                    if variant.individual not in individuals_in_pathway:
                        individuals_in_pathway.append(variant.individual)
                        
                #check size of individuals_in_pathway:
#                print 'individuals_in_pathway'
                if individuals_per_pathway_option == '>':
                    if len(individuals_in_pathway) >= individuals_per_pathway:
                        new_pathways.append(pathway)
            pathways = new_pathways
    else:    
        #show empty form
        form = PathwayAnalysisForm()
        variants = []
        summary = {}
        summary['has_variants'] = False
        #fake number just not to open accordion on view 
        summary['n_genes'] = 1000
        genes = []
        genes_hgmd = []
        genes_omim = []
        genes_cgd = []



    return render_to_response('pathway_analysis/analysis.html', 
        {'pathways':pathways,
        'form':form, 
        'summary':summary, 
        'query_string':query_string, 
        'filteranalysis':filteranalysis, 
        'filterconfigs':filterconfigs, 
        'genes':genes, 
        'genes_hgmd':genes_hgmd,
        'genes_omim':genes_omim,
        'genes_cgd':genes_cgd}, context_instance=RequestContext(request))

# #this method will quickly filter your data based on different criterias
# @login_required
# def get_filtered_variants(request):
#     query = {}
#     exclude = {}
#     summary = {}
#     args = []
#     query_string = request.META['QUERY_STRING']
#     new_query_string = []
#     for item in query_string.split('&'):
#         if not item.startswith('page'):
#              new_query_string.append(item)
#     query_string = "&".join(new_query_string)
    
#     print 'query string'
#     print query_string 
#     filteranalysis = FilterAnalysis.objects.all()
#     filterconfigs = FilterConfig.objects.all()
#     if query_string != [''] and query_string != '' :
#         if request.method == 'GET':
            
#             form = PathwayAnalysisForm(request.GET)
            
#             #CHR
#             chr = request.GET.get('chr', '')
#             if chr != '':
#                 query['chromossome'] = chr

#             #pos
#             pos = request.GET.get('pos', '')
#             if pos != '':
#                 pos = pos.split('-')
#                 query['pos__range'] = (pos[0], pos[1])

#             #snp_list
#             snp_list = request.GET.get('snp_list', '')
#             snp_list = snp_list.split('\r\n')
#             if snp_list[0] != u'':
#                 query['variant_id__in'] = snp_list
#             #exclude snp_list
#             exclude_snp_list = request.GET.get('exclude_snp_list', '')
#             exclude_snp_list = exclude_snp_list.split('\r\n')
#             if exclude_snp_list[0] != u'':
#                 args.append(~Q(variant_id__in=exclude_snp_list))
                
            
#             #Gene List
#             gene_list = request.GET.get('gene_list', '')
            
#             gene_list = gene_list.split('\r\n')
#             print 'gene_list'
            
#             if gene_list[0] != u'':
#                 safe_gene_list = []
#                 for row in gene_list:
#                     row = row.split(',')
#                     for item in row:
#                         safe_gene_list.append(item.strip())
#                 print safe_gene_list
#                 query['gene__in'] = safe_gene_list
            
#             #Gene group
#             genegroup_id = request.GET.get('genegroups', '')
# #            print 'GeneGroup'
# #            print genegroup_id
#             if genegroup_id != '':
#                 genegroup = GeneGroup.objects.get(id=int(genegroup_id))
#                 gene_list_group = genegroup.genes
#                 #make it possible for lines and commas #automagic
# #                print gene_list_group
#                 gene_groups = []
#                 gene_list_group = gene_list_group.split('\r\n')
#                 for row in gene_list_group:
#                     row = row.split(',')
#                     for item in row:
#                          item = item.strip()
#                          if item != "":
#                            gene_groups.append(item)
#                 query['gene__in'] = gene_groups
# #                print 'genegroup'
# #                print  gene_groups
            
#             #mutation_type
#             mutation_type = request.GET.get('mutation_type', '')
#             if mutation_type == 'homozygous':
#                 genotypes = ['0/0', './.', '0/1', '1/0', '0/2', '2/0']
#     #            exclude['genotype__in'] = genotypes 
#                 args.append(~Q(genotype__in=genotypes))
#     #            query['genotype'] = '1/1'
#             elif mutation_type == 'heterozygous':
#                 genotypes = ['0/0', './.', '1/1', '2/1', '1/2', '2/2']
#     #            ~Q(release_set__format='cd'
#                 args.append(~Q(genotype__in=genotypes))
#     #            exclude['genotype__in'] = genotypes
            
#             #exclude Gene List
#             exclude_gene_list = request.GET.get('exclude_gene_list', '')
#             exclude_gene_list = exclude_gene_list.split('\r\n')
#             if exclude_gene_list[0] != u'':
#     #            print 'exclude gene list'
#     #            print exclude_gene_list
#     #            print 'exclude gene list'
#                 args.append(~Q(gene__in=exclude_gene_list))
                
#     #            exclude['gene__in'] = exclude_gene_list
            
            
                
#             #Clinical
#             cln = request.GET.get('cln', '')
#             if cln == 'on':
#                 query['dbsnp_pm'] = 'Yes'    
                
#             #Variant type SNP EFF       
#             variant_type = request.GET.getlist('variant_type')
#             if len(variant_type) > 0:
#                 query['snp_eff__in'] = variant_type     
            
#             #DBSNP
#             dbsnp = request.GET.get('dbsnp', '')
#             if dbsnp == 'on':
#                 query['variant_id'] = "."
                
#             #1000genomes
#             genomes1000 = request.GET.get('genomes1000', '')
#             if genomes1000 != '':
#                 genomes1000_option = request.GET.get('genomes1000_option', '')
#                 if genomes1000_option == '<':
#                     #Also get null values
#                     args.append(Q(genomes1k_af__lte=float(genomes1000)) | Q(genomes1k_af__isnull=True))
#                 elif genomes1000_option == '>':
#                     args.append(Q(genomes1k_af__gte=float(genomes1000)) | Q(genomes1k_af__isnull=True))
            
#             #dbsnp Freq
#             dbsnp_frequency = request.GET.get('dbsnp_frequency', '')
#             if dbsnp_frequency != '':
#                 dbsnp_freq_option = request.GET.get('dbsnp_freq_option', '')
#                 if dbsnp_freq_option == '<':
#     #                query['dbsnp_gmaf__lte'] = float(dbsnp_frequency)
#                     args.append(Q(dbsnp_gmaf__lte=float(dbsnp_frequency)) | Q(dbsnp_gmaf__isnull=True))
#                 elif dbsnp_freq_option == '>':
#     #                query['dbsnp_gmaf__gte'] = float(dbsnp_frequency)
#                     args.append(Q(dbsnp_gmaf__gte=float(dbsnp_frequency)) | Q(dbsnp_gmaf__isnull=True))
            
#             #Exome Variation Server Freq
#             variationserver_frequency = request.GET.get('variationserver_frequency', '')
#             if variationserver_frequency != '':
#                 variationserver_option = request.GET.get('variationserver_option', '')
#                 if variationserver_option == '<':
#     #                print float(variationserver_frequency)*100.0
#     #                query['maf_total_esp5400__lte'] = float(variationserver_frequency)
#                     args.append(Q(ann_esp_maf__lte=float(variationserver_frequency)) | Q(ann_esp_maf__isnull=True))
#                 elif variationserver_option == '>':
#     #                query['maf_total_esp5400__gte'] = float(variationserver_frequency)
#                     args.append(Q(ann_esp_maf__gte=float(variationserver_frequency)) | Q(ann_esp_maf__isnull=True))
            
#             #SIFT
#             sift = request.GET.get('sift', '')
#             if sift != '':
#                 #Clinical
#                 sift_exclude = request.GET.get('sift_exclude', '')
#                 if sift_exclude == 'on':
#                     sift_flag = True
#                 else:
#                     sift_flag = False
                    
#                 sift_option = request.GET.get('sift_option', '')
#                 if sift_option == '<':
#     #                query['sift__lte'] = float(sift)
#                     if sift_flag:
#                         args.append(Q(sift__lte=float(sift)))
#                     else:
#                         args.append(Q(sift__lte=float(sift)) | Q(sift__isnull=True))
#                 elif sift_option == '>':
#                     if sift_flag:
#                         args.append(Q(sift__gte=float(sift)))
#                     else:
#                         args.append(Q(sift__gte=float(sift)) | Q(sift__isnull=True))
            
#             #POLYPHEN
#             polyphen = request.GET.get('polyphen', '')
#             if polyphen != '':
#                 polyphen_exclude = request.GET.get('polyphen_exclude', '')
#                 if polyphen_exclude == 'on':
#                     polyphen_flag = True
#                 else:
#                     polyphen_flag = False
                    
#                 polyphen_option = request.GET.get('polyphen_option', '')
#                 if polyphen_option == '<':
#                     if polyphen_flag:
#                         args.append(Q(polyphen__lte=float(polyphen)))
#                     else:
#                         args.append(Q(polyphen__lte=float(polyphen)) | Q(polyphen__isnull=True))
#                 elif polyphen_option == '>':
#                     if polyphen_flag:
#                         args.append(Q(polyphen__gte=float(polyphen)))
#                     else:
#                         args.append(Q(polyphen__gte=float(polyphen)) | Q(polyphen__isnull=True))
                    
#             #DBSNP Build
#             dbsnp_build = request.GET.get('dbsnp_build', '')
#             if dbsnp_build != '':
#                 dbsnp_option = request.GET.get('dbsnp_option', '')
#                 if dbsnp_option == '<':
#                     build_list = range(1, int(dbsnp_build))
#                     T2 = [str(x) for x in build_list]
#     #                print T2
#                     T2.append('')
#                     query['dbsnp_build__in'] = T2
#                 elif dbsnp_option == '>':
#                     build_list = range(int(dbsnp_build), 138)
#                     T2 = [str(x) for x in build_list]
#     #                print T2
#                     T2.append('')
#                     query['dbsnp_build__in'] = T2
#             #Read Depth
#             read_depth = request.GET.get('read_depth', '')
#             if read_depth != '':
#                 read_depth_option = request.GET.get('read_depth_option', '')
#                 if read_depth_option == '<':
#                     args.append(Q(read_depth__lte=int(read_depth)))
#                 elif read_depth_option == '>':
#                     args.append(Q(read_depth__gte=int(read_depth)))
            
#             #Functional class
#             func_class = request.GET.getlist('func_class')
#             if len(func_class) > 0:
#                 query['snp_eff_functional_class__in'] = func_class
#             #IMPACT           
#             impact = request.GET.getlist('impact')
#             if len(impact) > 0:
#                 query['impact__in'] = impact
            
#             print 'Exclude individuals'
#             #exclude individuals
#             exclude_individuals = request.GET.getlist('exclude_individuals')
#             print exclude_individuals
            
#             #exclude groups append to individuals
#             exclude_groups = request.GET.getlist('exclude_groups')
#             print exclude_groups
#             exclude_individuals_list = []
#             if len(exclude_groups) > 0:
#                 for group_id in exclude_groups:
#                     group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
#                     for individual in group_individuals:
#                          exclude_individuals_list.append(unicode(str(individual)))
#                     print exclude_individuals_list
                    
#             exclude_individuals_list = exclude_individuals_list + exclude_individuals
#             print exclude_individuals_list
            
#             exclude_individuals_variants = {}
            
#             #exclude variants from individuals
#             if len(exclude_individuals_list) > 0:
                
#                 #for all individuals in exclude list
#                 for individual in exclude_individuals_list:
#                     individual_variants = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values('chromossome', 'pos', 'genotype')
#                     for variant in individual_variants:
#                         id = '%s-%s' % (variant['chromossome'], variant['pos'])                        
#                         if id in exclude_individuals_variants:
#                             exclude_individuals_variants[id][variant['genotype']] = 0
#                         else:
#                             exclude_individuals_variants[id] = {}
#                             exclude_individuals_variants[id][variant['genotype']] = 0
                            
#             #INDIVIDUALS (MUST BE THE LAST ONE...) WHY ???
#             individuals = request.GET.getlist('individuals')
            
#             groups = request.GET.getlist('groups')
#             print groups
#             individuals_list = []
#             if len(groups) > 0:
#                 for group_id in groups:
#                     group_individuals = get_object_or_404(Group, pk=group_id).members.values_list('id', flat=True)
#                     for individual in group_individuals:
#                          individuals_list.append(unicode(str(individual)))
#                     print individuals_list
                    
#             individuals_list = individuals_list + individuals
            
#             if len(individuals_list) > 0:
                 
#                 query['individual__id__in'] = individuals_list
#                 #exclude variants from individuals_list
#                 if len(exclude_individuals_list) > 0:
#     #                print 'now excluding variants from exclude_individuals'
#                     variants_ids = []
#                     #get variants from all select individuals    
#                     variants = Variant.objects.filter(*args, **query).exclude(**exclude).values('id', 'chromossome', 'pos', 'genotype')
#     #                print 'variants from selected individuals'
#     #                print variants.count() 
                    
#                     for variant in variants:
#                         id = '%s-%s' % (variant['chromossome'], variant['pos'])
#                         if id in exclude_individuals_variants:
#                             if not variant['genotype'] in exclude_individuals_variants[id]:
#                                 variants_ids.append(variant['id'])
#                         else:
#                             variants_ids.append(variant['id'])
#     #                print len(variants_ids)
#                     query['pk__in'] = variants_ids
                
                
#                 # OPTION variants per gene
#                 variants_per_gene = request.GET.get('variants_per_gene')
#                 if variants_per_gene != '':
                    
#                     variants_per_gene = int(variants_per_gene)
#     #                print 'Variants per gene'
#                     variants_per_gene_option = request.GET.get('variants_per_gene_option', '')
#                     genes_exclude_list = []
#                     for individual in individuals_list:
#     #                    print individual
#                         individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values('gene').exclude(gene="").annotate(count=Count('gene')).distinct() #.aggregate(Count('gene', distinct=True))
#     #                    print len(individual_genes)
#                         if variants_per_gene_option == '>':    
#                             for gene in individual_genes:
#                                 if gene['count'] < variants_per_gene:
#                                     genes_exclude_list.append(gene['gene'])
#                         elif variants_per_gene_option == '<':                    
#                             for gene in individual_genes:
#                                 if gene['count'] > variants_per_gene:
#                                     genes_exclude_list.append(gene['gene'])
#                         elif variants_per_gene_option == '=':
#                             for gene in individual_genes:
#                                 if gene['count'] != variants_per_gene:
#                                     genes_exclude_list.append(gene['gene'])
#                     #remove variants without gene name
#                     genes_exclude_list.append('')
                    
#                     if 'gene__in' in exclude:
#                         exclude['gene__in'] = exclude['gene__in'] + genes_exclude_list
#                     else:
#                         exclude['gene__in'] = genes_exclude_list
                    
#                 #option GENES in COMMON
#                 genes_in_common = request.GET.get('genes_in_common', '')
#                 if genes_in_common == 'on':
                    
#                     #get all genes from individual
#                     individual_gene_list = []
#                     for individual in individuals_list:
#     #                    print individual
#                         individual_genes = Variant.objects.filter(individual__id=individual, *args, **query).exclude(**exclude).values_list('gene', flat=True).exclude(gene="").distinct()
#                         individual_genes = set(list(individual_genes))
#                         individual_gene_list.append(individual_genes)
#     #                print 'intersection'
#                     genes_in_common_list = set.intersection(*individual_gene_list)
#     #                print len(genes_in_common_list)
#     #                print 'genes in common'
#     #                print genes_in_common_list 
#     #                new_genes_in_common = []
#     #                if 'gene__in' in exclude:
#     #                    for item in genes_in_common_list:
#     #                        if item not in exclude['gene__in']:
#     #                            new_genes_in_common.append(item)
#     #                    genes_in_common_list = new_genes_in_common
                    
#                     query['gene__in'] = genes_in_common_list#genes_in_common_list
                    
#     #        print args
#     #        print query
#     #        print exclude
#             variants = Variant.objects.filter(*args, **query).exclude(**exclude).order_by('gene')
#             return variants





from django.forms import ModelForm
from individuals.models import *
from django import forms
from variants.models import *


class IndividualForm(ModelForm):


    vcf_file = forms.CharField(max_length=600, required=False)

    class Meta:
        model = Individual
        fields = '__all__'
        #'user', 'shared_with_users', , 'is_featured', 'is_public', 'vcf_file', 'vcf_header', 'status', 'n_variants', 'n_lines', 'creation_date', 'modified_date', 'annotation_time', 'insertion_time', 'insertion_time_mongo'

class GroupForm(ModelForm):
    class Meta:
        model = Group
        fields = '__all__'


class ControlGroupForm(ModelForm):
    class Meta:
        model = ControlGroup
        fields = '__all__'


class ComparisonForm(forms.Form):

    individual_one = forms.ModelChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)
    individual_two = forms.ModelChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)
    # individual_two = forms.ChoiceField(choices=[(x.id, x.name) for x in Individual.objects.all()], required=False)
    read_depth = forms.CharField(max_length=50, required=False, label="Read Depth")

    def __init__(self, user=None, *args, **kwargs):
        
        super(ComparisonForm, self).__init__(*args, **kwargs)

        if user.is_staff:
            
            self.fields['individual_one'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)
            self.fields['individual_two'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.all().order_by('id'), required=False)

        elif not user.is_authenticated:
            
            self.fields['individual_one'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False)
            self.fields['individual_two'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=None).order_by('id'), required=False)
        else:
            print('user', user)
            self.fields['individual_one'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=user).order_by('id'), required=False)
            self.fields['individual_two'] = forms.ModelMultipleChoiceField(queryset=Individual.objects.filter(user=user).order_by('id'), required=False)
    

            
class BrowserForm(forms.Form):
    
    chr = forms.CharField(max_length=50, required=False)
    pos = forms.CharField(max_length=50, required=False)
    
    snp_id = forms.CharField(max_length=50, required=False)
    snp_list = forms.CharField(widget=forms.Textarea, required=False)
    
    gene = forms.CharField(max_length=50, required=False)
    gene_list = forms.CharField(widget=forms.Textarea, required=False)

    CHOICES = [(x, x.replace('_', ' ')) for x in ['CDS', 'CHROMOSOME_LARGE DELETION', 'CODON_CHANGE', 'CODON_INSERTION', 'CODON_CHANGE_PLUS CODON_INSERTION', 'CODON_DELETION', 'CODON_CHANGE_PLUS CODON_DELETION', 'DOWNSTREAM', 'EXON', 'EXON_DELETED', 'FRAME_SHIFT', 'GENE', 'INTERGENIC', 'INTERGENIC_CONSERVED', 'INTRAGENIC', 'INTRON', 'INTRON_CONSERVED', 'MICRO_RNA', 'NON_SYNONYMOUS_CODING', 'NON_SYNONYMOUS_START', 'NON_SYNONYMOUS_STOP', 'RARE_AMINO_ACID', 'SPLICE_SITE_ACCEPTOR', 'SPLICE_SITE_DONOR', 'SPLICE_SITE_REGION', 'SPLICE_SITE_BRANCH', 'SPLICE_SITE_BRANCH_U12', 'STOP_LOST', 'START_GAINED', 'START_LOST', 'STOP_GAINED', 'SYNONYMOUS_CODING', 'SYNONYMOUS_START', 'SYNONYMOUS_STOP', 'TRANSCRIPT', 'REGULATION', 'UPSTREAM', 'UTR_3_PRIME', 'UTR_3_DELETED', 'UTR_5_PRIME', 'UTR_5_DELETED']]
    
    snp_eff = forms.MultipleChoiceField(choices=CHOICES, required=False)

    # CHOICES = [(x[0], x[0]) for x in Variant.objects.values_list('snpeff_func_class').distinct()]

    CHOICES = [(x, x.replace('_', ' ')) for x in ['NONE', 'SILENT', 'MISSENSE', 'NONSENSE']]

    func_class = forms.MultipleChoiceField(choices=CHOICES, required=False)
    
    CHOICES = [(x, x.replace('_', ' ')) for x in ['HIGH', 'MODERATE', 'MODIFIER', 'LOW']]

    impact = forms.MultipleChoiceField(choices=CHOICES, required=False)
    
    cln = forms.BooleanField(required=False)



# -*- coding: utf-8 -*-
"""JSONResponse views for model widgets."""
from __future__ import absolute_import, unicode_literals

from django.core import signing
from django.core.signing import BadSignature
from django.http import Http404, JsonResponse
from django.views.generic.list import BaseListView

from .cache import cache
from .conf import settings


class AutoResponseView(BaseListView):
    """
    View that handles requests from heavy model widgets.

    The view only supports HTTP's GET method.
    """

    def get(self, request, *args, **kwargs):
        """
        Return a :class:`.django.http.JsonResponse`.

        Example::

            {
                'results': [
                    {
                        'text': "foo",
                        'id': 123
                    }
                ],
                'more': true
            }

        """
        self.widget = self.get_widget_or_404()
        self.term = kwargs.get('term', request.GET.get('term', ''))
        self.object_list = self.get_queryset()
        context = self.get_context_data()
        return JsonResponse({
            'results': [
                {
                    'text': self.widget.label_from_instance(obj),
                    'id': obj.pk,
                }
                for obj in context['object_list']
                ],
            'more': context['page_obj'].has_next()
        })

    def get_queryset(self):
        """Get QuerySet from cached widget."""
        return self.widget.filter_queryset(self.term, self.queryset)

    def get_paginate_by(self, queryset):
        """Paginate response by size of widget's `max_results` parameter."""
        return self.widget.max_results

    def get_widget_or_404(self):
        """
        Get and return widget from cache.

        Raises:
            Http404: If if the widget can not be found or no id is provided.

        Returns:
            ModelSelect2Mixin: Widget from cache.

        """
        field_id = self.kwargs.get('field_id', self.request.GET.get('field_id', None))
        if not field_id:
            raise Http404('No "field_id" provided.')
        try:
            key = signing.loads(field_id)
        except BadSignature:
            raise Http404('Invalid "field_id".')
        else:
            cache_key = '%s%s' % (settings.SELECT2_CACHE_PREFIX, key)
            widget_dict = cache.get(cache_key)
            if widget_dict is None:
                raise Http404('field_id not found')
            if widget_dict.pop('url') != self.request.path:
                raise Http404('field_id was issued for the view.')
        qs, qs.query = widget_dict.pop('queryset')
        self.queryset = qs.all()
        widget_dict['queryset'] = self.queryset
        widget_cls = widget_dict.pop('cls')
        return widget_cls(**widget_dict)



from django import forms
from .models import Genome1kSample

class Genomes1kForm(forms.Form):
    sample = forms.ModelChoiceField(
        queryset=Genome1kSample.objects.all().order_by('name'),
    )




# -*- coding: utf-8 -*-
# Create your views here.
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, render
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.shortcuts import get_object_or_404
# Imaginary function to handle an uploaded file.
#from somewhere import handle_uploaded_file
from django.contrib import messages
from individuals.models import Individual
from variants.models import Variant

from diseases.models import *
from genes.models import Gene as Gene2
from django.db.models import Q

# import urllib.request, urllib.parse, urllib.error

from django.views.generic import ListView
from django.urls import reverse
from django.contrib.admin.views.decorators import staff_member_required
# from bs4 import BeautifulSoup
# import bs4
import os


#class based
#@login_required
class DiseaseListView(ListView):
    model = Disease      #implies -> queryset = models.Car.objects.all()
    paginate_by = 100  #and that's it !!
    context_object_name = "diseases"
    
    def get_queryset(self, **kwargs):
        name = self.request.GET.get('name', '')    
        return Disease.objects.filter(Q(name__icontains=name)|Q(gene_names__icontains=name))

    # @method_decorator(login_required)
    # def dispatch(self, *args, **kwargs):
    #     return super(DiseaseListView, self).dispatch(*args, **kwargs)
    
    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(DiseaseListView, self).get_context_data(**kwargs)
        # Add in a QuerySet of all the books
        diseases = context['object_list']
        gene_list = []
        for disease in diseases:
            genes = disease.gene_names.split(',')
            for gene in genes:
                gene_list.append(gene.strip())
        context['filter_query'] = "%2C".join(gene_list)
        return context
    
    def post(self, request, *args, **kwargs):
        #get list of diseases
        diseases_ids = request.POST.getlist('diseases')
        diseases = Disease.objects.filter(id__in=diseases_ids)
        #create list of genes
        gene_list = []
        for disease in diseases:
            genes = disease.gene_names.split(',')
            for gene in genes:
                parsed_gene = gene.strip()
                if parsed_gene not in gene_list:
                    gene_list.append(parsed_gene)
        #redirect to filter analysis page
        gene_list_url = '?gene_list=%s' % ("%2C".join(gene_list))
        #return redirect('/filter_analysis/?gene_list=%s' % ("%2C".join(gene_list)))
        # return redirect(reverse('filter_analysis', args=['some_value']))
        return redirect(reverse('filter_analysis')+gene_list_url)
        # return redirect('filter_analysis', gene_list='%s' % ("%2C".join(gene_list)))

class HGMDGeneListView(ListView):
    model = HGMDGene      #implies -> queryset = models.Car.objects.all()
    paginate_by = 100  #and that's it !!
    context_object_name = "genes"
    
    def get_queryset(self, **kwargs):
        name = self.request.GET.get('name', '')    
        return HGMDGene.objects.filter(Q(name__icontains=name)|Q(gene_names__icontains=name))

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(HGMDGeneListView, self).dispatch(*args, **kwargs)
    
    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(HGMDGeneListView, self).get_context_data(**kwargs)
        # Add in a QuerySet of all the books
        genes = context['object_list']
        # gene_list = []
        # for disease in diseases:
        #     genes = disease.gene_names.split(',')
        #     for gene in genes:
        #         gene_list.append(gene.strip())
        # context['filter_query'] = "%2C".join(gene_list)
        return context
    
    def post(self, request, *args, **kwargs):
        #get list of diseases
        diseases_ids = request.POST.getlist('diseases')
        diseases = Disease.objects.filter(id__in=diseases_ids)
        #create list of genes
        gene_list = []
        for disease in diseases:
            genes = disease.gene_names.split(',')
            for gene in genes:
                gene_list.append(gene.strip())
        #redirect to filter analysis page
        gene_list_url = '?gene_list=%s' % ("%2C".join(gene_list))
        #return redirect('/filter_analysis/?gene_list=%s' % ("%2C".join(gene_list)))
        # return redirect(reverse('filter_analysis', args=['some_value']))
        return redirect(reverse('filter_analysis')+gene_list_url)
        # return redirect('filter_analysis', gene_list='%s' % ("%2C".join(gene_list)))
                

import os
@staff_member_required
def populate(request):

    print(os.getcwd())
    
    morbidmap = open('data/omim/morbidmap', 'r')
    for line in morbidmap:
#        print line
        line = line.strip().split('|')
        name = line[0]
        omim_id = line[2]
        chr_location = line[3]
        gene_names = line[1]
        disease = Disease.objects.create(name=name, omim_id=omim_id, chr_location=chr_location, gene_names=gene_names)
        disease.save()
        genes = line[1].split(',')
        
        
        gene_officialname = genes[0].strip()
        
        chr_candidate = line[3].split('q')[0]
        chr_candidate2 = line[3].split('p')[0]
        if len(chr_candidate) < len(chr_candidate2):
            chromossome = chr_candidate
        else:
            chromossome = chr_candidate2
        #add gene relation with disease
        if gene_names != '':
            #check if gene exists otherwise create gene
            try:
                gene = Gene.objects.get(names=gene_names, official_name=gene_officialname, chromossome=chromossome)
                gene.diseases.add(disease)
                gene.save()
            except Gene.DoesNotExist:
                gene = Gene.objects.create(names=gene_names, official_name=gene_officialname, chromossome=chromossome)
                gene.save()
                gene.diseases.add(disease)
                gene.save()
                
         
            #same for genes_Gene
            #check if gene exists for each alternative name of gene
            gene_names = gene_names.strip().split(',')
            try:
                gene = Gene2.objects.get(symbol=gene_officialname)
                gene.diseases.add(disease)
                gene.save()
            except Gene2.DoesNotExist:
                print('gene symbol doesnt exists in HGNC')
                print(gene_officialname)
                print(gene_names)
#   for gene_symbol in gene_names:
#       

        
        
                    #gene = Genes_Gene.objects.create(names=gene_names, official_name=gene_officialname, chromossome=chromossome)
                    #gene.save()
                    #gene.diseases.add(disease)
                    #gene.save()
                    
                 
                    
                    
    morbidmap.close()
    print('finish filling diseases')
    messages.add_message(request, messages.INFO, "Finished filling diseases")
    return redirect('/databases/')

@staff_member_required
def populate_genes(request):
    refgene = open('data/refgene', 'r')
    genes_coords = {}
    for line in refgene:
        if not line.startswith('#'):
            line = line.split('\t')
            gene_name = line[12]
            gene = {}
            gene['chr_location'] = line[2]
            gene['strand'] = line[3]
            #fix for coordinated + 1
            
            gene['txStart'] = int(line[4]) + 1
            gene['txEnd'] = int(line[5]) + 1
            gene['cdsStart'] = int(line[6]) + 1
            gene['cdsEnd'] = int(line[7]) + 1
            gene['exonCount'] = line[8]
            exons_start =  []
            for exon in line[9].split(','):
                if exon != '': 
                    exons_start.append(int(exon) + 1)
            gene['exonStarts'] = ','.join(map(str, exons_start))
            exons_end =  []
            for exon in line[10].split(','):
                if exon != '': 
                    exons_end.append(int(exon) + 1)
                    
            gene['exonEnds'] = ','.join(map(str, exons_end))
            genes_coords[gene_name] = gene
    print('finish loading refseq!!') 
    print(len(genes_coords))
    genes = Gene.objects.all()
    for gene in genes:
#        print gene.official_name
        if gene.official_name in genes_coords:
            gene.chr_location = genes_coords[gene.official_name]['chr_location']
            gene.strand = genes_coords[gene.official_name]['strand']
            gene.transcription_start = genes_coords[gene.official_name]['txStart']
            gene.transcription_end = genes_coords[gene.official_name]['txEnd']
            gene.cds_start = genes_coords[gene.official_name]['cdsStart']
            gene.cds_end = genes_coords[gene.official_name]['cdsEnd']
            gene.exons_count = genes_coords[gene.official_name]['exonCount']
            gene.exons_start = genes_coords[gene.official_name]['exonStarts']
            gene.exons_end = genes_coords[gene.official_name]['exonEnds']
            gene.save()
    print('finish filling genes!!')    
    messages.add_message(request, messages.INFO, "Finished filling genes: %s " % (len(genes_coords)))
    return redirect('/databases/')
 

@login_required
def list(request):
    diseases = Disease.objects.all()
    return render(request, 'diseases/list.html', {'diseases': diseases})
 

@login_required
def view(request, disease_id):
    disease = get_object_or_404(Disease, pk=disease_id)
    gene = disease.gene_set.all()[0]
    # print 'gene', gene.official_name
    # individuals = Individual.objects.all()
    individuals_variants = Variant.objects.filter(gene=gene.official_name)
    # individuals_variants = []
    # for individual in individuals:
    #     individual_variants = Variant.objects.filter(individual=individual, gene=gene.official_name)
    #     individuals_variants.append(individual_variants)
#    individual_variants.query.group_by = ['individual_id']
#    results = query.execute_sql()
#    individuals = Individual.objects.all()
#    for individual in individuals:
#        individual_variants = Variant.objects.find(filter(date__range=["2011-01-01", "2011-01-31"]))
    
    
    return render(request, 'diseases/view.html', {'disease': disease, 'variants': individuals_variants})
    
@staff_member_required
def populate_hgmd_genes(request):
    hgmd_dir = '/projects/hgmd/hgmd'
    listing = os.listdir(hgmd_dir)
    genes_list = []
    for genes_file in listing:
        if genes_file.endswith('html'):
            print(genes_file)
            gfile = '%s/%s' % (hgmd_dir, genes_file)
            soup = BeautifulSoup(open(gfile))
            
            tables = soup.table.findAll('table')
            for row in tables[3]:
                # print row.contents[2].string
                if row.contents[2].string != 'Gene description':
                    # # print row.contents
                    # for item in row.contents:
                    #   print item
                    # print row.contents[0].a.string
                    symbol = row.contents[0].a.string
                    aliases = ''
                    if len(row.contents[0].font.contents) > 1:
                        aliases = row.contents[0].font.contents[1]

                    description = row.contents[2].contents[0]
                    description_aliases = ''
                    if len(row.contents[2].contents[1].font.contents) > 1:
                        description_aliases = row.contents[2].contents[1].font.contents[1]

                    location = row.contents[3].string
                    n_mutations = int(row.contents[5].contents[0].contents[0].contents[0]['value'])


                    gene = HGMDGene(
                        symbol=symbol, 
                        aliases=aliases, 
                        description=description,
                        description_aliases = description_aliases,
                        location=location,
                        n_mutations = n_mutations,
                        )
                    genes_list.append(gene)
    HGMDGene.objects.bulk_create(genes_list)
    messages.add_message(request, messages.INFO, "Finished filling HGMD genes: %s " % (len(genes_list)))
    return redirect('/databases/')
                    




@staff_member_required
def populate_hgmd_mutations(request):
    hgmd_dir = '/projects/hgmd/hgmd/genes/'
    listing = os.listdir(hgmd_dir)
    mutation_list = []
    mutation_counter = 0

    for genes_file in listing:
        if genes_file.endswith('html'):
            # print genes_file
            gfile = '%s/%s' % (hgmd_dir, genes_file)
            # gfile = '%s/C9orf72.html' % (hgmd_dir)
            soup = BeautifulSoup(open(gfile), "html.parser")
            if len(mutation_list) > 5000:
                # print '5000'
                print('mutation_counter', mutation_counter)
                HGMDMutation.objects.bulk_create(mutation_list)
                mutation_list = []

            
            # print soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents[1].a.string
            #soup.prettify()
            # for 
            gene_name = soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents[1].a.string
            # print gene_name
            # gene = gene_name
            gene = HGMDGene.objects.filter(symbol__exact=str(gene_name))[0]
            # print gene

            #loop of each type of mutation
            
            # print 'Hello antes de loop'

            # print soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents
            index = 0
            for row in soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents:
                # print row
                if index >3:
                    if row.name == 'h3':
                        # print 'found mutation type'
                        
                        mutation_type = str(row.contents[0].split(' : ')[0]).strip()
                        
                        # print mutation_type
                        #create a new mutation to insert at the database
                        
                        table_with_mut = soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents[index+1]
                        
                        rows = table_with_mut.find_all("tr")[1:]
                        # print 'number of mutations: %s' % len(rows)
                        
                        for mutation in rows:
                            mutation_counter += 1
                            mutation_record = HGMDMutation()
                            mutation_record.gene = gene
                            mutation_record.mutation_type = mutation_type
                            # print 'mutation'
                            mutation_record.acession = mutation.contents[0].contents[0].contents[0].contents[0].contents[0]['value']
                            if mutation_type == 'Missense/nonsense':
                                # print mutation
                                mutation_record.codon_change = mutation.contents[1].string
                                mutation_record.aa_change = mutation.contents[2].string
                                mutation_record.hgvs_nucleotide = mutation.contents[3].string
                                mutation_record.hgvs_protein = mutation.contents[4].string
                                phenotype = mutation.contents[5].string
                                mutation_record.reference = mutation.contents[6].a
                                extra_information = mutation.contents[7]
                                mutation_record.extras = extra_information.contents[0]

                            elif mutation_type == 'Splicing':
                                
                                mutation_record.splicing_mutation = mutation.contents[1].string
                                mutation_record.hgvs_nucleotide = mutation.contents[2].string

                                phenotype = mutation.contents[3].string
                                mutation_record.reference = mutation.contents[4].a
                                extra_information = mutation.contents[5]
                                mutation_record.extras = extra_information.contents[0]

                            elif mutation_type == 'Regulatory':
                                
                                # regulatory_sequence = " ".join(mutation.contents[1](text=True))
                                mutation_record.regulatory_sequence = " ".join(mutation.contents[1](text=True))
                                phenotype = mutation.contents[2].string
                                mutation_record.reference = mutation.contents[3].a
                                extra_information = mutation.contents[4]
                                mutation_record.extras = extra_information.contents[0]

                            elif mutation_type == 'Small deletions':
                                
                                mutation_record.deletion_sequence = " ".join(mutation.contents[1](text=True))
                                mutation_record.hgvs_nucleotide = mutation.contents[2].string
                                phenotype = mutation.contents[3].string
                                mutation_record.reference = mutation.contents[4].a
                                extra_information = mutation.contents[5]
                                mutation_record.extras = extra_information.contents[0]

                            elif mutation_type == 'Small insertions':
                                
                                mutation_record.insertion_sequence = "".join(mutation.contents[1](text=True))
                                mutation_record.hgvs_nucleotide = mutation.contents[2].string
                                phenotype = mutation.contents[3].string
                                mutation_record.reference = mutation.contents[4].a
                                extra_information = mutation.contents[5]
                                mutation_record.extras = extra_information.contents[0]
                            elif mutation_type == 'Small indels':
                                
                                mutation_record.deletion_sequence = "".join(mutation.contents[1](text=True))
                                mutation_record.insertion_sequence = mutation.contents[2]
                                mutation_record.hgvs_nucleotide = mutation.contents[3].string
                                phenotype = mutation.contents[4].string
                                mutation_record.reference = mutation.contents[5].a
                                extra_information = mutation.contents[6]
                                mutation_record.extras = extra_information.contents[0]

                            elif mutation_type == 'Gross deletions':
                                
                                mutation_record.dna_level = mutation.contents[1].string
                                mutation_record.description = mutation.contents[2].string
                                phenotype = mutation.contents[3].string
                                mutation_record.reference = mutation.contents[4].a
                                extra_information = mutation.contents[5]
                                mutation_record.extras = extra_information.contents[0]

                            elif mutation_type == 'Gross insertions':
                                
                                mutation_record.dna_level = mutation.contents[1].string
                                mutation_record.insertion_duplication = mutation.contents[2].string
                                mutation_record.description = mutation.contents[3].string
                                phenotype = mutation.contents[4].string
                                mutation_record.reference = mutation.contents[5].a
                                extra_information = mutation.contents[6]
                                mutation_record.extras = extra_information.contents[0]
                            elif mutation_type == 'Complex rearrangements':
                                
                                mutation_record.description = mutation.contents[1].string
                                phenotype = mutation.contents[2].string
                                mutation_record.reference = mutation.contents[3].a
                                extra_information = mutation.contents[4]
                                mutation_record.extras = extra_information.contents[0]
                            elif mutation_type == 'Repeat variations':
                                
                                mutation_record.amplified_sequence = mutation.contents[1].string
                                mutation_record.location = mutation.contents[2].string
                                mutation_record.normal_range = mutation.contents[3].string
                                mutation_record.pathological_range = mutation.contents[4].string
                                phenotype = mutation.contents[5].string
                                mutation_record.reference = mutation.contents[6].a
                                extra_information = mutation.contents[7]
                                mutation_record.extras = extra_information.contents[0]



                            else:
                                   print(mutation_type)
                                   die()

                            # print extra_information
                            # print '#loop over extra'
                            # print extra_information
                            for item in extra_information.contents[0]:
                                # print 'item'
                                # print type(item)
                                # print item
                                # if type(item)==bs4.element.NavigableString:
                                    # print 'item'
                                    # print item
                                
                                if type(item)==bs4.element.Tag:
                                    # print 'item'
                                    # print item
                                    # print item['title']
                                    if item.name == 'img':
                                        if item['title'].startswith('Coordinate Chr'):
                                            mutation_record.coordinate = item['title'].split('Coordinate Chr ')[1]
                                            mutation_record.chromossome = mutation_record.coordinate.split(': ')[0]
                                            mutation_record.position = mutation_record.coordinate.split(': ')[1]
                                        # if item['title'] == 'Disease causing mutation':
                                        #     print 'Foudn disease causing mutation!!!'
                                        #     mutation_record.dm_mutation = True

                                    elif item.name == 'a':
                                        # print item.img['alt']
                                        if item.img['alt'].startswith('dbSNP ID'):
                                            mutation_record.rsid = item.img['alt'].split('dbSNP ID: ')[1]
                                            # print rsid
                                        else:
                                            print(item)
                                            print('algo estranho!')
                                            die()
                            #time to insert mutation at the database


                            #save one record and die
                            #get phenotype to save
                            try:
                                pheno = HGMDPhenotype.objects.get(name=phenotype.encode('utf-8'))
                            except HGMDPhenotype.DoesNotExist:
                                #create one
                                pheno = HGMDPhenotype(name=phenotype.encode('utf-8'))
                                pheno.save()
                                gene.diseases.add(pheno)
                                
                            # pheno.genes.add(gene)
                            mutation_record.phenotype = pheno
                            # mutation_record.save()
                            # die()

                            mutation_list.append(mutation_record)
                index += 1

    # print 'mutation_counter', mutation_counter
    # print 'inseringo Bulk!'
    HGMDMutation.objects.bulk_create(mutation_list)
    messages.add_message(request, messages.INFO, "Finished filling HGMD mutations:")
    return redirect('/databases/')

  # hgmd_dir = '/projects/hgmd/hgmd/genes/'
  # listing = os.listdir(hgmd_dir)
  # mutation_list = []
  # count = 0
  # gene_count = 0
  # for genes_file in listing:
  #     if genes_file.endswith('html'):
        
  #       gene_count += 1

  #       if gene_count == 2:
  #           die()

  #       print 'genes_file', genes_file
  #       print 'gene_count', gene_count

  #       gfile = '%s/%s' % (hgmd_dir, genes_file)
  #       # gfile = '%s/C9orf72.html' % (hgmd_dir)
  #       soup = BeautifulSoup(open(gfile), "html.parser")

  #       if len(mutation_list) > 5000:
  #           print '5000'
  #           HGMDMutation.objects.bulk_create(mutation_list)
  #           mutation_list = []
  #       # print soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents[1].a.string
  #       #soup.prettify()
  #       # for 
  #       gene_name = soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents[1].a.string
  #       print 'gene_name', gene_name
  #       # gene = gene_name
  #       gene = HGMDGene.objects.filter(symbol__exact=str(gene_name))[0]
  #       # print gene

  #       #loop of each type of mutation
        
  #       print 'Hello antes de loop'
  #       #create a pattern to get mutattions
  #       print soup#.contents[1].contents[2].contents[1].contents[1].contents[2]#.contents
  #       die()
  #       index = 0
  #       for row in soup.contents[1].contents[2].contents[1].contents[1].contents[2].contents:
          
          
  #         if index > 3:
  #           if row.name == 'h3':
  #             print 'found mutation type'
              
  #             mutation_type = str(row.contents[0].split(' : ')[0]).strip()
              
  #             print mutation_type
  #             #create a new mutation to insert at the database
              
  #             table_with_mut = soup.contents[1].contents[2].contents[1].contents[1].contents[2]#.contents[index+1]
  #             print 'table mut', table_with_mut
  #             die()

  #             rows = table_with_mut.find_all("tr")[1:]
  #             print 'number of mutations: %s' % len(rows)
              
  #             for mutation in rows:
  #               mutation_record = HGMDMutation()
  #               mutation_record.gene = gene
  #               mutation_record.mutation_type = mutation_type
  #               # print 'mutation'
  #               mutation_record.acession = mutation.contents[0].contents[0].contents[0].contents[0].contents[0]['value']
  #               if mutation_type == 'Missense/nonsense':
  #                 # print mutation
  #                 mutation_record.codon_change = mutation.contents[1].string
  #                 mutation_record.aa_change = mutation.contents[2].string
  #                 mutation_record.hgvs_nucleotide = mutation.contents[3].string
  #                 mutation_record.hgvs_protein = mutation.contents[4].string
  #                 phenotype = mutation.contents[5].string
  #                 mutation_record.reference = mutation.contents[6].a
  #                 extra_information = mutation.contents[7]
  #                 mutation_record.extras = extra_information.contents[0]

  #               elif mutation_type == 'Splicing':
                  
  #                 mutation_record.splicing_mutation = mutation.contents[1].string
  #                 mutation_record.hgvs_nucleotide = mutation.contents[2].string

  #                 phenotype = mutation.contents[3].string
  #                 mutation_record.reference = mutation.contents[4].a
  #                 extra_information = mutation.contents[5]
  #                 mutation_record.extras = extra_information.contents[0]

  #               elif mutation_type == 'Regulatory':
                  
  #                 # regulatory_sequence = " ".join(mutation.contents[1](text=True))
  #                 mutation_record.regulatory_sequence = " ".join(mutation.contents[1](text=True))
  #                 phenotype = mutation.contents[2].string
  #                 mutation_record.reference = mutation.contents[3].a
  #                 extra_information = mutation.contents[4]
  #                 mutation_record.extras = extra_information.contents[0]

  #               elif mutation_type == 'Small deletions':
                  
  #                 mutation_record.deletion_sequence = " ".join(mutation.contents[1](text=True))
  #                 mutation_record.hgvs_nucleotide = mutation.contents[2].string
  #                 phenotype = mutation.contents[3].string
  #                 mutation_record.reference = mutation.contents[4].a
  #                 extra_information = mutation.contents[5]
  #                 mutation_record.extras = extra_information.contents[0]

  #               elif mutation_type == 'Small insertions':
                  
  #                 mutation_record.insertion_sequence = "".join(mutation.contents[1](text=True))
  #                 mutation_record.hgvs_nucleotide = mutation.contents[2].string
  #                 phenotype = mutation.contents[3].string
  #                 mutation_record.reference = mutation.contents[4].a
  #                 extra_information = mutation.contents[5]
  #                 mutation_record.extras = extra_information.contents[0]
  #               elif mutation_type == 'Small indels':
                  
  #                 mutation_record.deletion_sequence = "".join(mutation.contents[1](text=True))
  #                 mutation_record.insertion_sequence = mutation.contents[2]
  #                 mutation_record.hgvs_nucleotide = mutation.contents[3].string
  #                 phenotype = mutation.contents[4].string
  #                 mutation_record.reference = mutation.contents[5].a
  #                 extra_information = mutation.contents[6]
  #                 mutation_record.extras = extra_information.contents[0]

  #               elif mutation_type == 'Gross deletions':
                  
  #                 mutation_record.dna_level = mutation.contents[1].string
  #                 mutation_record.description = mutation.contents[2].string
  #                 phenotype = mutation.contents[3].string
  #                 mutation_record.reference = mutation.contents[4].a
  #                 extra_information = mutation.contents[5]
  #                 mutation_record.extras = extra_information.contents[0]

  #               elif mutation_type == 'Gross insertions':
                  
  #                 mutation_record.dna_level = mutation.contents[1].string
  #                 mutation_record.insertion_duplication = mutation.contents[2].string
  #                 mutation_record.description = mutation.contents[3].string
  #                 phenotype = mutation.contents[4].string
  #                 mutation_record.reference = mutation.contents[5].a
  #                 extra_information = mutation.contents[6]
  #                 mutation_record.extras = extra_information.contents[0]
  #               elif mutation_type == 'Complex rearrangements':
                  
  #                 mutation_record.description = mutation.contents[1].string
  #                 phenotype = mutation.contents[2].string
  #                 mutation_record.reference = mutation.contents[3].a
  #                 extra_information = mutation.contents[4]
  #                 mutation_record.extras = extra_information.contents[0]
  #               elif mutation_type == 'Repeat variations':
                  
  #                 mutation_record.amplified_sequence = mutation.contents[1].string
  #                 mutation_record.location = mutation.contents[2].string
  #                 mutation_record.normal_range = mutation.contents[3].string
  #                 mutation_record.pathological_range = mutation.contents[4].string
  #                 phenotype = mutation.contents[5].string
  #                 mutation_record.reference = mutation.contents[6].a
  #                 extra_information = mutation.contents[7]
  #                 mutation_record.extras = extra_information.contents[0]

  #               else:
  #                  print mutation_type
  #                  die()

  #               # print extra_information
  #               # print '#loop over extra'
  #               # print extra_information
  #               extra_information = extra_information.find_all()
  #               # print extra_information

  #               for item in extra_information:#.contents[0]
  #                 # print 'item'
  #                 # print type(item)
  #                 # print item
  #                 # if type(item)==bs4.element.NavigableString:
  #                   # print 'item'
  #                   # print item
                  
  #                 if type(item)==bs4.element.Tag:
  #                   # print 'item'
  #                   # print item
  #                   # print item['title']
  #                   if item.name == 'img':
  #                     # print 'img'
  #                     # print item['title'] 
  #                     if item['title'].startswith('Coordinate Chr'):
  #                       mutation_record.coordinate = item['title'].split('Coordinate Chr ')[1]
  #                       mutation_record.chromossome = mutation_record.coordinate.split(': ')[0]
  #                       mutation_record.position = mutation_record.coordinate.split(': ')[1]
  #                     if item['title'] == 'Disease causing mutation':
  #                        # print 'Foudn disease causing mutation!!!'
  #                        mutation_record.dm_mutation = True

  #                   elif item.name == 'a':
  #                     # print item.img['alt']
  #                     if item.img['alt'].startswith('dbSNP ID'):
  #                       mutation_record.rsid = item.img['alt'].split('dbSNP ID: ')[1]
  #                       # print rsid
  #                   else:
  #                     print item
  #                     print 'algo estranho!'
  #                     die()
  #               #time to insert mutation at the database
  #               #save one record and die
  #               #get phenotype to save
  #               try:
  #                 pheno = HGMDPhenotype.objects.get(name=phenotype.encode('utf-8').strip())
  #               except HGMDPhenotype.DoesNotExist:
  #                 #create one
  #                 pheno = HGMDPhenotype(name=phenotype.encode('utf-8').strip())
  #                 pheno.save()
  #                 gene.diseases.add(pheno)

                  
  #               # pheno.genes.add(gene)
  #               mutation_record.phenotype = pheno
  #               # mutation_record.save()
  #               # die()
  #               count += 1
  #               print count
  #               mutation_list.append(mutation_record)
  #         index += 1
  #       #end of first gene



from django.core.management.base import BaseCommand, CommandError
from databases.models import VariSNP
import os

class Command(BaseCommand):
    help = 'Populate VariSNP'

    def handle(self, *args, **options):
        self.stdout.write('Populate VariSNP')
        file = open('data/neutral_snv_2014-06-24.csv', 'r')
        header = file.readline()

        for line in file:
            variant = line.split('\t')
            snpid = 'rs'+variant[0]
            snp = VariSNP(dbsnp_id=snpid)
            snp.save()
        print("Finished Inserting Data")


            
            
        #get all individuals
        # individuals = Individual.objects.all().order_by('id')
        # for individual in individuals:
        # 	print individual.id
        # 	print individual.variants_file
        # 	#get user folder
        # 	filepath = str(individual.variants_file).split('/')
        # 	user = filepath[1]
        # 	print user
        # 	#now copy to destiny
        # 	orig_path = 'genomes/'
        # 	path = 'backup'
        # 	d = '%s/%s' % (path, user)
        # 	#create user dir
        # 	if not os.path.exists(d):
        # 		os.makedirs(d)
        # 	command = 'cp %s/%s %s/' % (orig_path, individual.variants_file, d)
        # 	os.system(command)





# Create your tasks here
from __future__ import absolute_import, unicode_literals
from celery import shared_task

from .models import Project, ProjectFile, ProjectSample 
from settings.models import S3Credential
import boto3


@shared_task
def import_project_files_task(project_id):
    print('project', project_id)
    project = Project.objects.get(pk=project_id)
    samples = ProjectSample.objects.filter(project=project)
    files = ProjectFile.objects.filter(project=project)
    # print(samples)
    for sample in samples:
        sample.n_files = 0
        sample.n_fastqs = 0
        sample.n_bams = 0
        sample.n_vcfs = 0
        sample.files.clear()
        #get all files from location
        if sample.location != '':
            for file in files:
                add_file = False
                location = file.location
                if sample.location in file.location:
                    if sample.prefix != '':
                        if sample.prefix in file.location:
                            add_file = True
                    else:
                        add_file = True
                    if add_file:
                        sample.files.add(file)
                        sample.n_files += 1
                        if location.endswith('.fq.gz') or location.endswith('.fastq.gz'):
                            sample.n_fastqs += 1
                        if location.endswith('.bam'):
                            sample.n_bams += 1
                        if location.endswith('.vcf.gz') or location.endswith('.vcf'):
                            sample.n_vcfs += 1
        sample.save()
        #match if prefix is available
        #add them all to sample
        #save and be happy :D



from django.db import models
from datetime import datetime

class Worker(models.Model):

    name = models.CharField(max_length=30)
    provider = models.CharField(max_length=30)
    type = models.CharField(max_length=30)
    n_tasks = models.IntegerField(default=0, null=True, blank=True)
    status = models.CharField(max_length=30)
    current_status = models.TextField(null=True, blank=True)
    worker_id = models.CharField(max_length=30)
    ip = models.CharField(max_length=30)
    creation_date = models.DateTimeField(auto_now_add=True, null=True, blank=True)
    modified_date = models.DateTimeField(auto_now=True, null=True, blank=True)
    started = models.DateTimeField(null=True)
    finished = models.DateTimeField(null=True)
    execution_time = models.TimeField(null=True)


from django.shortcuts import render, redirect
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.urls import reverse_lazy
from django.core.management import call_command

from files.models import File
from django.contrib.auth.decorators import login_required

from tasks.tasks import compress_file
from .tasks import check_file
from tasks.models import Task

from django.contrib.auth.mixins import LoginRequiredMixin

from django.shortcuts import get_object_or_404, redirect
from collections import Counter
from django.db.models import Q

from django.http import HttpResponse
#for upload
from .forms import UploadForm
from django.utils.text import slugify
import os
from django.conf import settings
import json

# from .forms import FileForm


@login_required
def index(request):

    query  = ''
    args = []

    if request.method == 'POST':

        # form = FileForm(request.POST)
        # if form.is_valid():
        #     pass

        # print(request.POST)
        files = request.POST.getlist('files')
        action = request.POST['action']
        query = request.POST['query']
        order_string ='sample'
        # print('query', query)
        if action == 'analysis':
            request.session['files'] = files
            return redirect('analysis-create')
        
        for file_id in files:

            # file = File.objects.get(pk=file_id)
            if request.user.is_staff:
                file = get_object_or_404(File, pk=file_id)
            else:
                file = get_object_or_404(File, pk=file_id, user=request.user)

            if action == "delete":
                file.delete()
            else:
                task_manifest = {}
                task_manifest['file'] = file.id
                task_manifest['action'] = action
                task = Task(user=request.user)
                
                task.manifest = task_manifest
                task.status = 'new'
                task.action = action
                task.user = request.user
                task.save()

                if action == "check":
                    check_file.delay(task.id)                
                if action == "compress":
                    compress_file.delay(task.id)
                file.status = 'scheduled'
                file.save()

        status = request.POST.getlist('status')
        
        print('status', status)

        if len(status[0]) > 0:
            args.append(Q(status__in=status))

        extension = request.POST.getlist('extension')

        if len(extension[0]) > 0:
            print('extension',extension)
            args.append(Q(extension__in=extension))

        print('args', args)

    else:
        # form = FileForm()
        print(request.GET)
        if 'orderby' in request.GET:
            orderby = request.GET['orderby']
            order = request.GET['order'][0]
            if order == 'desc':
                order_string = '-{}'.format(orderby)
            else:
                order_string = orderby
        else:
            order_string = 'name'

    # print(order_string, 'order_string')
    
    if request.user.is_staff:#status='scheduled' size=0
        # files = File.objects.filter(location__icontains=query, *args).order_by(order_string)#size
        files = File.objects.filter(location__regex=query, *args).order_by(order_string)#size
        
    else:
        files = File.objects.filter(user=request.user).order_by(order_string)
    #sample__isnull=True
    # files = File.objects.filter(*args).order_by('-id')

    files_summary = {}
    files_summary['status'] = dict(Counter(files.values_list('status', flat=True)))
    files_summary['file_type'] = dict(Counter(files.values_list('file_type', flat=True)))
    files_summary['extension'] = dict(Counter(files.values_list('extension', flat=True)))
    # files_summary['total_size'] = sum(files.values_list('size', flat=True))
    # files_summary = 0
    filetypes = []
    n_files = []
    for filetype in files_summary['extension']:
        # print(filetype, files_summary['extension'][filetype])
        filetypes.append(filetype)
        n_files.append(files_summary['extension'][filetype])
    files_summary['filetypes'] = filetypes
    files_summary['n_files'] = n_files


    context = {
        # 'form': form,
        'query':query,
        'files':files,
        'n_files':len(files),
        'files_summary':files_summary
    }

    return render(request, 'files/index.html', context)

@login_required
def view(request, file_id):

    file = File.objects.get(pk=file_id)
    print(dir(file))
    context = {
    'file':file
    }
    return render(request, 'files/view.html', context)


@login_required
def import_files(request):
    print('Hello World')
    
    # 


    return redirect('files-index')

def response_mimetype(request):
    if "application/json" in request.META['HTTP_ACCEPT']:
        return "application/json"
    else:
        return "text/plain"


class JSONResponse(HttpResponse):
    """JSON response class."""
    def __init__(self,obj='',json_opts={},mimetype="application/json",*args,**kwargs):
        content = json.dumps(obj,**json_opts)
        super(JSONResponse,self).__init__(content,mimetype,*args,**kwargs)

def upload(request):
    if request.method == 'POST':
        form = UploadForm(request.POST, request.FILES)
        
        if form.is_valid():
            
            if request.user.is_authenticated:
                file = File.objects.create(user=request.user, status='new')
            else:
                file = File.objects.create(user=None, status='new')

            file.local_file = request.FILES.get('file')
            
            print('file')
            print(request.FILES.get('file'))

            filename = file.local_file.name.split('.')
            new_filename = []
            for tag in filename:
                new_filename.append(slugify(tag))

            file.local_file.name = ".".join(new_filename)

            

            # print('file.location ', file.location)

            #get name from inside vcf file
            #clean this...
            file.name= str(os.path.splitext(file.local_file.name)[0]).replace('.vcf','').replace('.gz','').replace('.rar','').replace('.zip','').replace('._',' ').replace('.',' ')

            # file.shared_with_groups = form.cleaned_data['shared_with_groups']
            # file.shared_with_groups.set(form.cleaned_data['shared_with_groups'])
            
            file.save()
            
            f = file.local_file

            #fix permissions
            #os.chmod("%s/genomes/%s/" % (settings.BASE_DIR, file.user), 0777)

            if request.user.is_authenticated:
                file_path = "%s/media/%s/%s" % (settings.BASE_DIR, slugify(file.user), file.id)
            else:
                file_path = "%s/media/public/%s" % (settings.BASE_DIR, file.id)
            os.chmod(file_path, 0o777)

            file.location = '/'+file.local_file.url

            # AnnotateVariants.delay(file.id)
            
            task_manifest = {}
            task_manifest['file'] = file.id
            task_manifest['action'] = 'check'
            task = Task(user=request.user)
            
            task.manifest = task_manifest
            task.status = 'new'
            task.action = 'check'
            task.save()

            check_file.delay(task.id)

            file.status = 'scheduled'
            file.save()

            data = {'files': [{'deleteType': 'DELETE', 'name': file.name, 'url': '', 'thumbnailUrl': '', 'type': 'image/png', 'deleteUrl': '', 'size': f.size}]}

            response = JSONResponse(data, mimetype=response_mimetype(request))
            response['Content-Disposition'] = 'inline; filename=files.json'
            return response
        else:
            print(form.errors)

    else:
        form = UploadForm()

    return render(request, 'files/upload.html', {'form':form})


class FileUpdate(LoginRequiredMixin, UpdateView):
    model = File
    fields = '__all__'

    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects

class FileDelete(LoginRequiredMixin, DeleteView):
    model = File
    success_url = reverse_lazy('files-index')
    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects

@login_required
def bulk_action(request):

    if request.method == 'POST':
        files = request.POST.getlist('files')
        action = request.POST['action']

        for file_id in files:


            # file = File.objects.get(pk=file_id)
            if request.user.is_staff:
                file = get_object_or_404(File, pk=file_id)
            else:
                file = get_object_or_404(File, pk=file_id, user=request.user)

            if action == "delete":
                file.delete()
            if action == "check":
                
                task_manifest = {}
                task_manifest['file'] = file.id
                task_manifest['action'] = action
                task = Task(user=request.user)
                
                task.manifest = task_manifest
                task.status = 'new'
                task.action = action
                task.user = request.user
                task.save()

                check_file.delay(task.id)

                file.status = 'scheduled'
                file.save()

    return redirect('files-index')

@login_required
def run_task(request):
    if request.method == 'GET':
        print(request.GET)
        if 'action' in request.GET:
            action = request.GET['action']
            file_id  = request.GET['file_id']

            file = File.get_object_or_404(pk=file_id)
            if action == "check":
                
                task_manifest = {}
                task_manifest['file'] = file.id
                task_manifest['action'] = action
                task = Task(user=request.user)
                
                task.manifest = task_manifest
                task.status = 'new'
                task.action = action
                task.user = request.user
                task.save()

                check_file.delay(task.id)

                file.status = 'scheduled'
                file.save()
            
    return redirect('files-index')

@login_required
def run_task(request):
    if request.method == 'GET':
        print(request.GET)
        if 'action' in request.GET:
            action = request.GET['action'][0]
            file_id  = request.GET['file_id'][0]
            print(action, file_id)
    return redirect('files-index')


from django.core.management.base import BaseCommand, CommandError
from files.models import File
from settings.models import S3Credential
import boto3
import os
from django.conf import settings
import time

class Command(BaseCommand):
    help = 'Import Files'

    def handle(self, *args, **options):
        print('Hello World Import Files')
        start_time = time.time()

        self.download_files()
        self.import_files()
        elapsed_time = time.time() - start_time
        print('Importing Files Took {}'.format(elapsed_time))
    def download_files(self):
        print('Download Files')
        file_list = open('%s/data/files/all_files.txt' % (settings.BASE_DIR), 'w')
        s3credentials = S3Credential.objects.all()
        for s3credential in s3credentials:
            print(s3credential.name)
            for bucket_name in s3credential.buckets.splitlines():
                session = boto3.Session(
                    aws_access_key_id=s3credential.access_key,
                    aws_secret_access_key=s3credential.secret_key
                )
                s3 = session.resource('s3')
                bucket = s3.Bucket(bucket_name)
                print(bucket)
                for key in bucket.objects.all():
                    if key.size != 0:
                        file = [str(key.last_modified), str(key.size), bucket.name, key.key]
                        file_list.writelines('%s\n' % ('\t'.join(file)))
        self.stdout.write(self.style.SUCCESS('Successfully downloaded files!'))

    def import_files(self):
        print('Import Files')
        File.objects.all().delete()
        file_list = open('%s/data/files/all_files.txt' % (settings.BASE_DIR))
        file_types = {}
        files = {}
        exclude_paths = []
        s3credentials = S3Credential.objects.all()
        for s3credential in s3credentials:
            # print()
            # print(s3credential.exclude_paths.splitlines())
            for item in s3credential.exclude_paths.splitlines():
                exclude_paths.append(item)

        n_files = 0
        for line in file_list:
            row = line.strip().split('\t')
            file = {
                'date': row[0],
                'size': row[1],
                'bucket': row[2],
                'location': row[3],
            }
            if not file['location'].startswith(tuple(exclude_paths)):
                full_path = 's3://' + file['bucket'] + '/' + file['location']
                n_files += 1
                file_name, file_extension = os.path.splitext(file['location'])
                if file_extension == '.gz':
                    file_name, file_extension = os.path.splitext(file_name)
                file['extension'] = file_extension
                if file_extension not in file_types:
                    file_types[file_extension] = []
                file_types[file_extension].append(full_path)
                files[full_path] = file
        #
        # print(n_files)
        print('Summary')
        print('Number of files: {}'.format(n_files))
        print('Number of file types: {}'.format(len(file_types)))
        extensions = ['.fastq', '.bam', '.vcf']
        file_objs = []
        for extension in extensions:
            for file in file_types[extension]:
                # print(file)


                file_name, file_extension = os.path.splitext(file)
                if file_extension == '.gz':
                    file_name, file_extension = os.path.splitext(file_name)
                basename = os.path.basename(file_name)

                file_obj = File(
                    name=basename,
                    size=files[file]['size'],
                    last_modified=str(files[file]['date']),
                    file_type=file_extension.replace('.', ''),
                    location=file,
                )
                #print(file_obj,files[file])
                file_objs.append(file_obj)
                #file_obj.save()
        File.objects.bulk_create(file_objs)



from django.db import models
from django.forms import ModelForm
from .models import S3Credential

class S3CredentialForm(ModelForm):
    class Meta:
        model = S3Credential
        fields = '__all__'
    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(S3CredentialForm, self).__init__(*args, **kwargs)



from settings.models import S3Credential
import boto3
from celery import shared_task
from tasks.models import Task
from .models import File
from subprocess import check_output
import os

@shared_task()
def check_file(task_id):
    task = Task.objects.get(pk=task_id)

    task.status = 'started'
    task.save()

    manifest = task.manifest
    file_id = manifest['file']
    print('File ID', file_id)

    file = File.objects.get(pk=file_id)

    if file.location.startswith('ftp://'):
        print('ftp')
        print(file.location)

     
        file.name = os.path.basename(file.location)

        command = 'curl -sI {} | grep Content-Length'.format(file.location)
        output = check_output(command, shell=True)

        # print()
        size = output.split()[1]

        file.size = int(size)
        # file.human_size = human_size(file.size)



    elif file.location.startswith('http'):
        link = file.location
        print('link', link)
        print('link', link.strip())
        print('link', link.encode())

        file.name = os.path.basename(link)
        
        site = urllib.request.urlopen(link)
        file_size = site.info()['Content-Length']
        file.size = int(file_size)
        # file.human_size = human_size(int(file_size))
        
    elif file.location.startswith('/'):

        path = '/'.join(file.location.split('/')[:-1])

        command = 'ls -lah {}'.format(path)
        output = check_output(command, shell=True)
        file.last_output = output.decode('utf-8')

        filename, file_extension = os.path.splitext(file.location)

        if file_extension in ['.gz', '.zip', '.rar', 'bz2', '.7z']:
            compression = file_extension
            filename, file_extension  = os.path.splitext(filename)
            file_extension += compression

        file.extension = file_extension

        command = 'file {}'.format(file.location)
        output = check_output(command, shell=True)
        file.file_type = ' '.join(output.decode('utf-8').strip().split(' ')[1:])

        file.name = os.path.basename(file.location)
        # print(os.stat(file.location))
        # print(os.path.getsize(file.location))
        file.size = int(os.path.getsize(file.location))

    elif file.location.startswith('s3://'):

        # print('s3')
        # print(file.location)

     
        file.name = os.path.basename(file.location)

        clean_path = file.location.replace('s3://', '')
        split_path = clean_path.split('/', 1)
        bucket_name = split_path[0]
        prefix = split_path[1]
        # print(bucket_name,prefix)
        s3credentials = S3Credential.objects.all()

        for s3credential in s3credentials:
            # if clean_path.startswith(s3credential.buckets):
            if bucket_name in s3credential.buckets:
                #get all files from path
                session = boto3.Session(
                    aws_access_key_id=s3credential.access_key,
                    aws_secret_access_key=s3credential.secret_key
                )
                s3 = session.resource('s3')
                bucket = s3.Bucket(bucket_name)
                # print(bucket)
                # print('prefix',prefix)
                for key in bucket.objects.filter(Prefix=prefix):
                    if key.key == prefix:
                        file_name, file_extension = os.path.splitext(key.key)
                        if file_extension == '.gz':
                            file_name, file_extension = os.path.splitext(file_name)
                        
                        basename = os.path.basename(file_name)
                        file.name=basename
                        # print('key.size',key.size)
                        file.size=int(key.size)
                        file.last_modified=str(key.last_modified)
                        file.extension=file_extension.replace('.', '')

    print('File Name', file.name)
    file.status = 'checked'
    file.save()

    # if file.extension == '.vcf.gz' or file.extension == '.vcf':

    #     task_manifest = {}
    #     task_manifest['file'] = file.id
    #     task_manifest['action'] = 'import_vcf'
        
    #     import_task = Task()
    #     import_task.manifest = task_manifest
    #     import_task.status = 'new'
    #     import_task.action = 'import_vcf'
    #     import_task.save()
    #     import_vcf.delay(task.id)

    # task.status = 'done'
    # task.save()


@shared_task()
def import_vcf(task_id):
    print('importing a vcf file into the database')
    print(task_id)
    task = Task.objects.get(pk=task_id)

    task.status = 'started'
    task.save()
    
    #get vcf
    #get number of samples
    #create samples objects
    #add samples
    #add variants
    #add genotypes
    #kick annotation




# Generated by Django 2.1.1 on 2019-03-28 23:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('variants', '0003_variant_snpeff_codon_change'),
    ]

    operations = [
        migrations.AddField(
            model_name='variant',
            name='snpeff_aa_change',
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
    ]



from django.shortcuts import render

from .models import Sample, SampleGroup
from files.models import File
from tasks.models import Task
import json
from tasks.tasks import annotate_vcf, run_qc

from django.shortcuts import render, redirect, get_object_or_404
from django.db.models import Q
from django.contrib import messages

from files.tasks import import_vcf

# from sample_tracking_system.users.models import User
from django.contrib.auth.models import User

# from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

from django.db.models import F, Count

from django.contrib.auth.decorators import login_required
import csv

from django.http import HttpResponseRedirect, HttpResponse

import io

from samples.forms import SampleGroupForm

from django.views.generic import ListView
from django.views.generic.edit import DeleteView
from django.urls import reverse_lazy
from django.views.generic.detail import DetailView
from django.views.generic.edit import UpdateView

from django.contrib import messages


# Create your views here.
@login_required
def index(request):

    args = []

    if request.method == 'POST':
        action = request.POST['action']
        samples = request.POST.getlist('samples')
        # print('samples', samples)
        request.session['samples'] = samples
        return redirect('create_group')

    else:

        samples = Sample.objects.filter(*args).order_by('id')

    context = {
        'samples':samples,
    }

    return render(request, 'samples/index.html', context)


@login_required
def create_group(request):
    if request.method == 'POST':

        form = SampleGroupForm(request.POST)
        if form.is_valid():
            form.save()
            return redirect('sample_index')
    else:
        samples = request.session['samples']
        print('samples', samples)
        form = SampleGroupForm(initial = {'members': [1,2,3] })
        # form.members.initial = ['537']
    return render(request, 'samples/create_group.html', {'form': form})


def group_detail(request, pk):
    object = SampleGroup.objects.get(pk=pk)
    n_samples = object.members.count()

    context = {
        'object':object,
        'n_samples':n_samples
    }
    return render(request, 'samples/samplegroup_detail.html', context)

class SampleGroupList(ListView):
    model = SampleGroup


class SampleGroupDelete(DeleteView):
    model = SampleGroup
    success_url = reverse_lazy('samplegroup-list')

class SampleDetailView(DetailView):
    model = Sample

class SampleGroupUpdateView(UpdateView):
    model = SampleGroup
    fields = '__all__'

def sample_import_vcf(request, pk):
    sample = Sample.objects.get(pk=pk)
    messages.add_message(request, messages.SUCCESS, 'VCF from sample {} will be imported'.format(sample.name))
    import_vcf.delay(pk)
    return HttpResponseRedirect(request.META.get('HTTP_REFERER'))


@login_required
def bulk_action(request):

    if request.method == 'POST':
        samples = request.POST.getlist('samples')
        action = request.POST['action']
        print(action, samples)
        for sample_id in samples:
            if action == "import_vcf":
                import_vcf.apply_async(sample_id)
    return HttpResponseRedirect(request.META.get('HTTP_REFERER'))



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Worker',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('provider', models.CharField(max_length=30)),
                ('type', models.CharField(max_length=30)),
                ('n_tasks', models.IntegerField(blank=True, default=0, null=True)),
                ('status', models.CharField(max_length=30)),
                ('current_status', models.TextField(blank=True, null=True)),
                ('worker_id', models.CharField(max_length=30)),
                ('ip', models.CharField(max_length=30)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(auto_now=True, null=True)),
                ('started', models.DateTimeField(null=True)),
                ('finished', models.DateTimeField(null=True)),
                ('execution_time', models.TimeField(null=True)),
            ],
        ),
    ]



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('diseases', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CGDCondition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='CGDEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('GENE', models.TextField(blank=True)),
                ('HGNC_ID', models.TextField(blank=True)),
                ('ENTREZ_GENE_ID', models.TextField(blank=True)),
                ('INHERITANCE', models.TextField(blank=True)),
                ('AGE_GROUP', models.TextField(blank=True)),
                ('ALLELIC_CONDITIONS', models.TextField(blank=True)),
                ('COMMENTS', models.TextField(blank=True)),
                ('INTERVENTION_RATIONALE', models.TextField(blank=True)),
                ('REFERENCES', models.TextField(blank=True)),
                ('CONDITIONS', models.ManyToManyField(blank=True, to='genes.CGDCondition')),
            ],
        ),
        migrations.CreateModel(
            name='Gene',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hgnc_id', models.TextField(blank=True)),
                ('symbol', models.TextField(blank=True)),
                ('name', models.TextField(blank=True)),
                ('locus_group', models.TextField(blank=True)),
                ('locus_type', models.TextField(blank=True)),
                ('status', models.TextField(blank=True)),
                ('location', models.TextField(blank=True)),
                ('location_sortable', models.TextField(blank=True)),
                ('alias_symbol', models.TextField(blank=True)),
                ('alias_name', models.TextField(blank=True)),
                ('prev_symbol', models.TextField(blank=True)),
                ('prev_name', models.TextField(blank=True)),
                ('gene_family', models.TextField(blank=True)),
                ('gene_family_id', models.TextField(blank=True)),
                ('date_approved_reserved', models.TextField(blank=True)),
                ('date_symbol_changed', models.TextField(blank=True)),
                ('date_name_changed', models.TextField(blank=True)),
                ('date_modified', models.TextField(blank=True)),
                ('entrez_id', models.TextField(blank=True)),
                ('ensembl_gene_id', models.TextField(blank=True)),
                ('vega_id', models.TextField(blank=True)),
                ('ucsc_id', models.TextField(blank=True)),
                ('ena', models.TextField(blank=True)),
                ('refseq_accession', models.TextField(blank=True)),
                ('ccds_id', models.TextField(blank=True)),
                ('uniprot_ids', models.TextField(blank=True)),
                ('pubmed_id', models.TextField(blank=True)),
                ('mgd_id', models.TextField(blank=True)),
                ('rgd_id', models.TextField(blank=True)),
                ('lsdb', models.TextField(blank=True)),
                ('cosmic', models.TextField(blank=True)),
                ('omim_id', models.TextField(blank=True)),
                ('mirbase', models.TextField(blank=True)),
                ('homeodb', models.TextField(blank=True)),
                ('snornabase', models.TextField(blank=True)),
                ('bioparadigms_slc', models.TextField(blank=True)),
                ('orphanet', models.TextField(blank=True)),
                ('pseudogene_org', models.TextField(blank=True)),
                ('horde_id', models.TextField(blank=True)),
                ('merops', models.TextField(blank=True)),
                ('imgt', models.TextField(blank=True)),
                ('iuphar', models.TextField(blank=True)),
                ('kznf_gene_catalog', models.TextField(blank=True)),
                ('mamit_trnadb', models.TextField(blank=True)),
                ('cd', models.TextField(blank=True)),
                ('lncrnadb', models.TextField(blank=True)),
                ('enzyme_id', models.TextField(blank=True)),
                ('intermediate_filament_db', models.TextField(blank=True)),
                ('diseases', models.ManyToManyField(related_name='_gene_diseases_+', to='diseases.Disease')),
            ],
        ),
        migrations.CreateModel(
            name='GeneCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('domain', models.CharField(blank=True, max_length=255)),
                ('name', models.TextField(blank=True)),
                ('go', models.CharField(max_length=255)),
                ('definition', models.TextField(blank=True)),
                ('genes', models.ManyToManyField(to='genes.Gene')),
            ],
        ),
        migrations.CreateModel(
            name='GeneGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('genes', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='GeneList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255)),
                ('genes', models.TextField(blank=True)),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='GoTerm',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('goid', models.CharField(max_length=255)),
                ('name', models.CharField(max_length=255)),
                ('namespace', models.CharField(max_length=255)),
                ('level', models.CharField(max_length=255)),
                ('is_obsolete', models.BooleanField(default=True)),
                ('alt_ids', models.CharField(max_length=255)),
                ('children', models.ManyToManyField(blank=True, related_name='children_go', to='genes.GoTerm')),
                ('parents', models.ManyToManyField(blank=True, related_name='parents_go', to='genes.GoTerm')),
            ],
        ),
        migrations.CreateModel(
            name='Intervention',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Manifestation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('gene', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='genes.Gene')),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='genes.GeneCategory')),
            ],
        ),
        migrations.AddField(
            model_name='cgdentry',
            name='INTERVENTION_CATEGORIES',
            field=models.ManyToManyField(blank=True, to='genes.Intervention'),
        ),
        migrations.AddField(
            model_name='cgdentry',
            name='MANIFESTATION_CATEGORIES',
            field=models.ManyToManyField(blank=True, to='genes.Manifestation'),
        ),
    ]



from django.shortcuts import render
from .models import Task
from django.shortcuts import render, redirect, get_object_or_404
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.views.generic import DetailView

from django.urls import reverse_lazy
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

from tasks.tasks import task_run_task
from files.tasks import check_file
from django.db.models import Q
from collections import Counter

from files.models import File

@login_required
def index(request):

    query = request.GET.get('query', '')
    args = []

    if query != '':
        args.append(Q(name__icontains=query))

    if request.user.is_staff:
        tasks = Task.objects.filter(*args).order_by('-id')
    else:
        tasks = Task.objects.filter(*args, user=request.user).order_by('-id')

    tasks_summary = dict(Counter(tasks.values_list('status', flat=True)))

    n_tasks = len(tasks) 

    context = {
        'tasks':tasks,
        'n_tasks':n_tasks,
        'tasks_summary':tasks_summary
    }
    return render(request, 'tasks/index.html', context)

@method_decorator(login_required, name='dispatch')
class TaskDelete(DeleteView):

    model = Task

    success_url = reverse_lazy('tasks-index')
    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects


@login_required
def run_task(request, task_id):

    if request.user.is_staff:
        task = Task.objects.get(pk=task_id)
    else:
        task = Task.objects.get(pk=task_id, user=request.user)

    task_run_task.delay(task.id)

    # if task.action == "check":
    #     check_file.delay(task.id)
    # if task.action == "download":
    #     download_file.delay(task.id)

    task.status = 'scheduled'
    task.save()

    return redirect('tasks-view', task.id)

@method_decorator(login_required, name='dispatch')
class TaskDetail(DetailView):
    model = Task
    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects
    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super().get_context_data(**kwargs)
        # files = self.object.manifest['files']
        # context['input_files'] = File.objects.filter(pk__in=files)
        # context['output_files'] = File.objects.filter(task=self.object.id)
        return context
        
@login_required
def bulk_action(request):
    if request.method == 'POST':
        
        tasks = request.POST.getlist('tasks')
        action = request.POST['action']
        
        for task_id in tasks:

            if request.user.is_staff:
                task = Task.objects.get(pk=task_id)
            else:
                task = Task.objects.get(pk=task_id, user=request.user)
            
            #save to change modified date
            task.save()

            if action == "run":
                
                task_run_task.delay(task.id)
                # if task.action == "qc":
                #     task.status = 'scheduled'
                #     run_qc.delay(task.id)
                # if task.action == "check":
                #     task.status = 'scheduled'
                #     check_file.delay(task.id)
                # if task.action == "download":
                #     task.status = 'scheduled'
                #     download_file.delay(task.id)
                task.status = 'scheduled'
                task.save()

            if action == "delete":
                task.delete()

    return redirect('tasks-index')



from django.db import models
from django.contrib.auth.models import User, Group
from individuals.models import Individual
from files.models import File
from tasks.models import Task

class Project(models.Model):

    user = models.ForeignKey(User, editable=True, null=True, on_delete=models.CASCADE)
    individuals = models.ManyToManyField(Individual, blank=True)
    
    files = models.ManyToManyField(File, blank=True)
    tasks = models.ManyToManyField(Task, blank=True)

    name = models.CharField(max_length=30)
    description = models.TextField(null=True, blank=True)

    is_public = models.BooleanField(default=False)
    status = models.CharField(max_length=100, blank=True, editable=False)

    paths = models.TextField(null=True, blank=True)

    groups = models.ManyToManyField(Group, editable=True, related_name="project_groups", blank=True)
    members = models.ManyToManyField(User, editable=True, related_name="project_members", blank=True)

    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return "/projects/{}".format(self.id)

class ProjectFile(models.Model):

    user = models.ForeignKey(User, editable=False, null=True, on_delete=models.CASCADE)
    
    project = models.ForeignKey(Project, editable=False, null=True, on_delete=models.CASCADE)

    location = models.TextField(null=True, blank=True)
    name = models.CharField(max_length=300)
    size = models.BigIntegerField(null=True, blank=True)
    human_size = models.CharField(max_length=300)
    last_modified = models.DateTimeField(null=True, blank=True)
    file_type = models.TextField(null=True, blank=True)


    status = models.CharField(max_length=300)
    md5 = models.TextField(null=True, blank=True)
    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(null=True, blank=True)



class ProjectSample(models.Model):

    user = models.ForeignKey(User, editable=False, null=True, on_delete=models.CASCADE)
    project = models.ForeignKey(Project, editable=False, null=True, on_delete=models.CASCADE)
    files = models.ManyToManyField(ProjectFile, blank=True)

    name = models.CharField(max_length=300)
    alias = models.CharField(max_length=300)
    status = models.CharField(max_length=300, blank=True)
    location = models.TextField(null=True, blank=True)
    prefix = models.TextField(null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    comments = models.TextField(null=True, blank=True)

    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(auto_now=True, null=True, blank=True)
    last_modified = models.DateTimeField(null=True, blank=True)

    n_files = models.IntegerField(default=0,null=True, blank=True)
    n_fastqs = models.IntegerField(default=0,null=True, blank=True)
    n_bams = models.IntegerField(default=0,null=True, blank=True)
    n_vcfs = models.IntegerField(default=0,null=True, blank=True)

    def __str__(self):              # __unicode__ on Python 2
        return self.name



from django.views.generic.edit import DeleteView
from django.urls import reverse_lazy

from django.shortcuts import render, redirect
from django.contrib import messages

from workers.models import Worker
from workers.tasks import launch_worker, launch_workers, terminate_worker, install_worker, check_workers, update_worker

from subprocess import check_output, call

from django.contrib.auth.decorators import login_required
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.mixins import LoginRequiredMixin

# Create your views here.
@login_required
def index(request):
    workers = Worker.objects.all().order_by('-id')
    context = {'workers':workers}
    return render(request, 'workers/index.html', context)

@login_required
def launch(request):
    n = request.GET.get('n')
    if n:
        for i in range(0,int(n)):
            launch_worker.delay()
    else:
        launch_worker.delay()
    messages.success(request, 'Worker is being launched.')
    return redirect('worker-list')

@login_required
def terminate(request, pk):
    worker=Worker.objects.get(pk=pk)
    terminate_worker.delay(worker.id)
    messages.success(request, 'Worker is being terminated.')
    return redirect('worker-list')

@login_required
def install(request, pk):
    worker=Worker.objects.get(pk=pk)
    install_worker.delay(worker.id)
    messages.success(request, 'Worker is being installed.')
    return redirect('worker-list')

@login_required
def update(request, pk):
    worker=Worker.objects.get(pk=pk)
    update_worker.delay(worker.id)
    messages.success(request, 'Worker is being updated!')
    return redirect('worker-list')

class WorkerDelete(LoginRequiredMixin, DeleteView):
    model = Worker
    success_url = reverse_lazy('worker-list')


def action(request):
    if request.method == 'POST':
        print('Hello World')
        action = request.POST['action']
        workers = request.POST.getlist('select[]')
        
        # print(action, workers)
        for worker_id in workers:
            
            worker=Worker.objects.get(id=worker_id)

            if action == 'run':
                worker.current_status = 'queued'
                worker.save()
            elif action == 'update':
                update_worker.delay(worker.id)
            elif action == 'install':
                install_worker.delay(worker.id)
                worker.status = 'installing'
                worker.save()
            elif action == 'terminate':
                terminate_worker.delay(worker.id)
                worker.status = 'will be terminated'
                worker.save()
            elif action == 'delete':
                worker.delete()
        
        if action == 'check':                
            check_workers.delay()
            
                
    return redirect('worker-list')    


# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('files', '__first__'),
        ('auth', '0009_alter_user_last_name_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('individuals', '__first__'),
        ('tasks', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('description', models.TextField(blank=True, null=True)),
                ('is_public', models.BooleanField(default=False)),
                ('status', models.CharField(blank=True, editable=False, max_length=100)),
                ('paths', models.TextField(blank=True, null=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(blank=True, null=True)),
                ('files', models.ManyToManyField(blank=True, to='files.File')),
                ('groups', models.ManyToManyField(blank=True, related_name='project_groups', to='auth.Group')),
                ('individuals', models.ManyToManyField(blank=True, to='individuals.Individual')),
                ('members', models.ManyToManyField(blank=True, related_name='project_members', to=settings.AUTH_USER_MODEL)),
                ('tasks', models.ManyToManyField(blank=True, to='tasks.Task')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ProjectFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('location', models.TextField(blank=True, null=True)),
                ('name', models.CharField(max_length=300)),
                ('size', models.BigIntegerField(blank=True, null=True)),
                ('human_size', models.CharField(max_length=300)),
                ('last_modified', models.DateTimeField(blank=True, null=True)),
                ('file_type', models.TextField(blank=True, null=True)),
                ('status', models.CharField(max_length=300)),
                ('md5', models.TextField(blank=True, null=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(blank=True, null=True)),
                ('project', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='projects.Project')),
                ('user', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ProjectSample',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=300)),
                ('alias', models.CharField(max_length=300)),
                ('status', models.CharField(blank=True, max_length=300)),
                ('location', models.TextField(blank=True, null=True)),
                ('prefix', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('comments', models.TextField(blank=True, null=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(auto_now=True, null=True)),
                ('last_modified', models.DateTimeField(blank=True, null=True)),
                ('n_files', models.IntegerField(blank=True, default=0, null=True)),
                ('n_fastqs', models.IntegerField(blank=True, default=0, null=True)),
                ('n_bams', models.IntegerField(blank=True, default=0, null=True)),
                ('n_vcfs', models.IntegerField(blank=True, default=0, null=True)),
                ('files', models.ManyToManyField(blank=True, to='projects.ProjectFile')),
                ('project', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='projects.Project')),
                ('user', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import individuals.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ControlGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
                ('vcf_file', models.FileField(blank=True, help_text='File Format: VCF', max_length=600, upload_to=individuals.models.ControlGroup.get_upload_path)),
            ],
        ),
        migrations.CreateModel(
            name='ControlVariant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('index', models.TextField(db_index=True)),
                ('controlgroup', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='individuals.ControlGroup')),
            ],
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name='Individual',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
                ('is_featured', models.BooleanField(default=True)),
                ('is_public', models.BooleanField(default=False)),
                ('vcf_file', models.FileField(blank=True, help_text='File Format: VCF', max_length=600, upload_to=individuals.models.Individual.get_upload_path)),
                ('vcf_header', models.TextField(blank=True, null=True)),
                ('status', models.CharField(blank=True, editable=False, max_length=100)),
                ('n_variants', models.IntegerField(blank=True, editable=False, null=True)),
                ('n_lines', models.IntegerField(blank=True, editable=False, null=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(blank=True, null=True)),
                ('annotation_time', models.CharField(blank=True, max_length=200, null=True)),
                ('insertion_time', models.CharField(blank=True, max_length=200, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='UserGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
                ('members', models.ManyToManyField(blank=True, related_name='members', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='individual',
            name='shared_with_groups',
            field=models.ManyToManyField(blank=True, related_name='shared_with_groups', to='individuals.UserGroup'),
        ),
        migrations.AddField(
            model_name='individual',
            name='shared_with_users',
            field=models.ManyToManyField(blank=True, related_name='shared_with_users', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='individual',
            name='user',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='group',
            name='members',
            field=models.ManyToManyField(related_name='group_members', to='individuals.Individual'),
        ),
    ]



from django.core.management.base import BaseCommand, CommandError
from individuals.models import Individual
import os

class Command(BaseCommand):
    help = 'Extract Individuals'

    def handle(self, *args, **options):
        self.stdout.write('Hello World')
        #get all individuals
        individuals = Individual.objects.all().order_by('id')
        for individual in individuals:
        	print(individual.id)
        	print(individual.variants_file)
        	#get user folder
        	filepath = str(individual.variants_file).split('/')
        	user = filepath[1]
        	print(user)
        	#now copy to destiny
        	orig_path = 'genomes/'
        	path = 'backup'
        	d = '%s/%s' % (path, user)
        	#create user dir
        	if not os.path.exists(d):
        		os.makedirs(d)
        	command = 'cp %s/%s %s/' % (orig_path, individual.variants_file, d)
        	os.system(command)
            


from subprocess import run, check_output
import subprocess
from time import sleep


class SCW():

	def __init__(self):
		pass

	def main(self):
		print('main')

	def install():
		command = '''
		sudo apt-get update
		sudo apt-get -y upgrade
		wget https://storage.googleapis.com/golang/go1.9.2.linux-amd64.tar.gz
		sudo tar -xvf go1.9.2.linux-amd64.tar.gz
		sudo mv go /usr/local
		export GOROOT=/usr/local/go

		export GOPATH=$HOME/go

		export PATH=$GOPATH/bin:$GOROOT/bin:$PATH
		'''
		# command = 'GO15VENDOREXPERIMENT=1 go get -u github.com/scaleway/scaleway-cli/cmd/scw'
		run(command,shell=True)

	def launch(self, worker_type=None):
		# Create a new server but do not start it.
		result = {}
		# Options:

		command = 'scw --region=ams1 create --name="mendelmd_worker" ubuntu-xenial'#--volume=150GB --commercial-type=X64-15GB 
		output = run(command,shell=True, stdout=subprocess.PIPE).stdout.decode('utf-8').strip()


		short_id = output.split('-')[0]
		
		result['id'] = short_id

		command = 'scw --region=ams1 start {}'.format(short_id)
		output = run(command,shell=True, stdout=subprocess.PIPE).stdout.decode('utf-8').strip()

		flag = True
		while(flag):
			command = 'scw --region=ams1 ps -a'
			output = run(command,shell=True, stdout=subprocess.PIPE).stdout.decode('utf-8').splitlines()			
			for line in output:
				# print(line)
				row = line.split()
				# print(row)
				if row[0] == short_id:
					if row[5] == 'running':
						ip = row[6]
						result['ip'] = ip
						flag = False
					else:
						print('Waiting {} to start'.format(short_id))
						sleep(30)

		command = 'ssh-keygen -f ~/.ssh/known_hosts -R {}'.format(ip)
		print(command)

		output = run(command,shell=True, stdout=subprocess.PIPE).stdout.decode('utf-8')
		print(output)
		return(result)

	def terminate(self, id):
		command = 'scw --region=ams1 stop -t {}'.format(id)
		output = run(command,shell=True, stdout=subprocess.PIPE).stdout.decode('utf-8').splitlines()


		# scw ps
		# scw create ubuntu
		# scw start 5cf8058e
		# scw exec test01 echo "Hello world!"
		# scw help



# 1  FROM scaleway/ubuntu:lastest
# 2  MAINTAINER
# 3  
# 4  # Prepare rootfs for image-builder.
# 5  # This script prevent aptitude to run services when installed
# 6  RUN /usr/local/sbin/builder-enter
# 7  
# 8   # Your app
# 9  
# 10 # Install packages
# 11 RUN apt-get -q update && \
# 12  apt-get -y -qq upgrade && \
# 13  apt-get install -y -qq cowsay
# 14 
# 15 # Add local files from the patches directory
# 16 COPY ./patches/ /
# 17 
# 18 # End
# 19 
# 20 # Clean rootfs from image-builder.
# 21 # Revert the builder-enter script
# 22 RUN /usr/local/sbin/builder-leave		


from django import template

register = template.Library()


@register.filter
def get_genotype(variant):
    """Removes all values of arg from the given string"""
    genotype = variant.genotype
    if genotype == '0/0':
        return('(%s;%s)' % (variant.ref, variant.ref))
    elif genotype == '0/1':
        return('(%s;%s)' % (variant.ref, variant.alt))
    elif genotype == '1/0':
        return('(%s;%s)' % (variant.alt, variant.ref))
    elif genotype == '1/1':
        return('(%s;%s)' % (variant.alt, variant.alt))
    else:
        return ('ref:%s, alt:%s, genotype:%s' % (variant.ref, variant.alt, variant.genotype))



from django.conf.urls import *

from . import views

urlpatterns = [
    url(r'^$', views.index, name='apps-index'),
    url(r'^create/$', views.AppCreate.as_view(), name='apps-create'),
    url(r'update/(?P<pk>\d+)/$', views.AppUpdate.as_view(), name='apps-update'),
    
]


from django import template
register = template.Library()

@register.simple_tag
def dictKeyLookup(the_dict, key):
   # Try to fetch from the dict, and if it's not found return an empty string.
   return the_dict.get(key, '0')


@register.filter
def adjust_for_pagination(page, current):
    if (current - page) in [-1,-2,1,2]:
        return True
    else: 
        return False
register.filter('dictKeyLookup', dictKeyLookup)
register.filter('adjust_for_pagination', adjust_for_pagination)



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Disease',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('omim_id', models.CharField(max_length=255)),
                ('chr_location', models.CharField(max_length=255)),
                ('gene_names', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Gene',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('official_name', models.CharField(max_length=255)),
                ('chromossome', models.CharField(blank=True, max_length=255, null=True)),
                ('names', models.TextField(blank=True, null=True)),
                ('strand', models.CharField(blank=True, max_length=255, null=True)),
                ('chr_location', models.CharField(blank=True, max_length=255, null=True)),
                ('transcription_start', models.IntegerField(blank=True, null=True)),
                ('transcription_end', models.IntegerField(blank=True, null=True)),
                ('cds_start', models.IntegerField(blank=True, null=True)),
                ('cds_end', models.IntegerField(blank=True, null=True)),
                ('exons_count', models.CharField(blank=True, max_length=500, null=True)),
                ('exons_start', models.TextField(blank=True, null=True)),
                ('exons_end', models.TextField(blank=True, null=True)),
                ('diseases', models.ManyToManyField(to='diseases.Disease')),
            ],
        ),
        migrations.CreateModel(
            name='HGMDGene',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('symbol', models.CharField(max_length=255)),
                ('aliases', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('description_aliases', models.TextField(blank=True, null=True)),
                ('location', models.CharField(max_length=255)),
                ('n_mutations', models.IntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='HGMDMutation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('mutation_type', models.CharField(blank=True, max_length=255, null=True)),
                ('acession', models.CharField(blank=True, max_length=255, null=True)),
                ('reference', models.TextField(blank=True, null=True)),
                ('extras', models.TextField(blank=True, null=True)),
                ('rsid', models.CharField(blank=True, max_length=255, null=True)),
                ('dm_mutation', models.BooleanField(default=False)),
                ('coordinate', models.CharField(blank=True, max_length=100, null=True)),
                ('chromossome', models.CharField(blank=True, max_length=100, null=True)),
                ('position', models.CharField(blank=True, max_length=100, null=True)),
                ('codon_change', models.CharField(blank=True, max_length=255, null=True)),
                ('aa_change', models.CharField(blank=True, max_length=255, null=True)),
                ('hgvs_nucleotide', models.CharField(blank=True, max_length=100, null=True)),
                ('hgvs_protein', models.CharField(blank=True, max_length=100, null=True)),
                ('splicing_mutation', models.CharField(blank=True, max_length=255, null=True)),
                ('regulatory_sequence', models.TextField(blank=True, null=True)),
                ('deletion_sequence', models.TextField(blank=True, null=True)),
                ('insertion_sequence', models.TextField(blank=True, null=True)),
                ('dna_level', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.CharField(blank=True, max_length=255, null=True)),
                ('insertion_duplication', models.CharField(blank=True, max_length=255, null=True)),
                ('amplified_sequence', models.CharField(blank=True, max_length=255, null=True)),
                ('location', models.CharField(blank=True, max_length=255, null=True)),
                ('normal_range', models.CharField(blank=True, max_length=255, null=True)),
                ('pathological_range', models.CharField(blank=True, max_length=255, null=True)),
                ('gene', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='diseases.HGMDGene')),
            ],
        ),
        migrations.CreateModel(
            name='HGMDPhenotype',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.AddField(
            model_name='hgmdmutation',
            name='phenotype',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='diseases.HGMDPhenotype'),
        ),
        migrations.AddField(
            model_name='hgmdgene',
            name='diseases',
            field=models.ManyToManyField(to='diseases.HGMDPhenotype'),
        ),
    ]



import shlex
import subprocess

from django.core.management.base import BaseCommand
from django.utils import autoreload


def restart_celery():
    cmd = 'pkill celery'
    subprocess.call(shlex.split(cmd))
    cmd = 'celery worker -l info -A mendelmd -c 2' #-P solo
    subprocess.call(shlex.split(cmd))


class Command(BaseCommand):

    def handle(self, *args, **options):
        print('Starting celery worker with autoreload...')
        autoreload.main(restart_celery)



"""
WSGI config for mendelmd project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/howto/deployment/wsgi/
"""

import os

import site
import sys

#site.addsitedir('/projects/venv/lib/python3.5/site-packages')

#site.addsitedir('/projects/mendelmd_master')

#sys.path.append('/projects/venv/lib/python3.5/site-packages')
#sys.path.append('/projects/mendelmd_dev')
#sys.path.append('/home/ubuntu/projects/mendelmdenv/lib/python3.5/site-packages')
#sys.path.append('/home/ubuntu/projects/mendelmd/mendelmd_master')


os.environ.setdefault("DJANGO_SETTINGS_MODULE", "mendelmd.settings")

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()
